#ifndef kellett_non_halt_data_structures_header
#define kellett_non_halt_data_structures_header

/*Authored 2003    by  Owen Kellett,  UnderGraduate Research
(Advisor: Professor Bram van Heuveln)


This  file is part of the   "Generic Turing Machine Implementation for
Use in the Exploration of the Busy Beaver Problem".

The "Generic Turing Machine Implementation for  Use in the Exploration
of the Busy Beaver Problem" is free  software; you can redistribute it
and/or modify it under the terms of the  GNU General Public License as
published   by the Free Software Foundation;   either version 2 of the
License, or (at your option) any later version.

This programme is distributed in the hope that it  will be useful, but
WITHOUT    ANY  WARRANTY;  without   even  the    implied warranty  of
MERCHANTABILITY or FITNESS FOR  A  PARTICULAR  PURPOSE.  See  the  GNU
General Public License for more details.

You should have received   a copy of  the  GNU General Public  License
along  with  this   programme; if  not,  write   to the Free  Software
Foundation, Inc., 59  Temple Place, Suite  330, Boston, MA  02111-1307
USA (or visit http://www.fsf.org/licenses/licenses.html)

Created as part of the Busy Beaver Project:
Bram van Heuveln <heuveb@rpi.edu>
Boleshaw Szymanski <szymansk@cs.rpi.edu>
Selmer Bringsjord <selmer@rpi.edu>
Carlos Varela <cvarela@cs.rpi.edu>
Owen Kellett <kelleo@rpi.edu>
Shailesh Kelkar <kelkas@rpi.edu>
Kyle Ross <rossk2@rpi.edu>
Rensselaer Reasoning Group
Department of Cognitive Science
Department of Computer Science
Rensselaer Polytechnic Institute
Troy, New York 12180
U.S.A.
*/

#include <fstream>
#include <iostream>
#include <iterator>
#include <list>
#include <vector>
#include <map>

#include "infinite_tape.hpp"
#include "state.hpp"
#include "turing_machine.hpp"

typedef unsigned short d_type;

/**
 * @defgroup NonHaltDetectionDataStructures
 */

//Begin simpleLoopCheck data structures

/**
 * encapsulates a turing machine state and a current read head symbol.
 * @ingroup NonHaltDetectionDataStructures
 */
template <typename T>
class stateSymbolPair
{
public:
	/**
	 * default contructor.
	 * @author Owen Kellett
	 */
	stateSymbolPair()
	:stateIndicator(0), symbol(0)
	{}

	/**
	 * useful constructor.
	 * @post stateIndicator and symbol are initialized to parameters
	 * @param sI state of the pair
	 * @param s symbol of the pair
	 * @author Owen Kellett
	 */
	stateSymbolPair(typename std::vector<state<T> >::size_type sI, T s)
	:stateIndicator(sI), symbol(s)
	{}

	/**
	 * operator <
	 * @post whether lhs is less than rhs according to the state, and then the symbol
	 * @author Owen Kellett
	 */
	friend bool operator < <> (const stateSymbolPair<T> &lhs, const stateSymbolPair<T> &rhs);

	typename std::vector<state<T> >::size_type stateIndicator;
	T symbol;
};


/**
 * operator <
 * @post whether lhs is less than rhs according to the state, and then the symbol
 * @author Owen Kellett
 * @ingroup NonHaltDetectionDataStructures
 */
template <typename T>
bool operator < (const stateSymbolPair<T> &lhs, const stateSymbolPair<T> &rhs)
{
	if(lhs.stateIndicator == rhs.stateIndicator)
		return lhs.symbol < rhs.symbol;
	return lhs.stateIndicator < rhs.stateIndicator;
}

/**
 * maintains a snapshot of a turing machine tape include.
 * an instance of the tape
 * two iterators pointing to cells on the new instance representing left and right bounds of moves since the creation of this instance
 * two iterators pointint to the original tape (yes that's right) representing the same two bounds
 * @ingroup NonHaltDetectionDataStructures
 */
template <typename T>
class tapeBoundsTriple
{
public:
	/**
	 * standard constructor.
	 * @post tape is now a copy of t; leftBoundOriginal and rightBoundOriginal point to the bound cells of t; leftBound and rightBound point to the bound cells of the new instance tape
	 * @param t instance of the tape
	 * @param l left most read symbol on the tape thus far
	 * @param r right most read symbol on the tape thus far
	 * @author Owen Kellett
	 */
	tapeBoundsTriple(infinite_tape<T> &t, typename std::list<T>::iterator l, typename std::list<T>::iterator r)
	:tape(t), leftBoundOriginal(l), rightBoundOriginal(r)
	{
		typename std::list<T>::iterator lTempOriginal = t.begin();
		typename std::list<T>::iterator rTempOriginal = t.begin();
		leftBound = tape.begin();
		rightBound = tape.begin();

		//position the bounds on tape to match their positions on t
		while(lTempOriginal != leftBoundOriginal)
		{
			++lTempOriginal;
			++leftBound;
		}
		while(rTempOriginal != rightBoundOriginal)
		{
			++rTempOriginal;
			++rightBound;
		}
	}

	/**
	 * copy constructor.
	 * @author Owen Kellett
	 */
	tapeBoundsTriple(const tapeBoundsTriple<T> &rhs)
	:tape(rhs.tape), leftBoundOriginal(rhs.leftBoundOriginal), rightBoundOriginal(rhs.rightBoundOriginal)
	{
		typename std::list<T>::const_iterator lTempOriginal = rhs.tape.begin();
		typename std::list<T>::const_iterator rTempOriginal = rhs.tape.begin();
		leftBound = tape.begin();
		rightBound = tape.begin();

		//position the bounds on tape to match their positions on t
		while(lTempOriginal != rhs.leftBound)
		{
			++lTempOriginal;
			++leftBound;
		}
		while(rTempOriginal != rhs.rightBound)
		{
			++rTempOriginal;
			++rightBound;
		}
	}

	infinite_tape<T> tape;
	typename std::list<T>::iterator leftBound;
	typename std::list<T>::iterator rightBound;

	typename std::list<T>::iterator leftBoundOriginal;
	typename std::list<T>::iterator rightBoundOriginal;
};

//end simpleLoopCheck data structures

//begin backTrack data structures

/**
 * represents a free standing transition for a turing machine.
 * @ingroup NonHaltDetectionDataStructures
 */
template <typename T>
class owenTransition
{
public:
	/**
	 * default contructor.
	 * @author Owen Kellett
	 */
	owenTransition()
	{}

	/**
	 * standard constructor.
	 * @post this is representative of a transition given the parameters
	 * @param newRead read symbol of the transition
	 * @param newWrite newWrite symbol to be written to the tape
	 * @param newNextState next state to transition to
	 * @param newOriginalState original state of the transition
	 * @param newMove direction to move on the tape
	 * @author Owen Kellett
	 */
	owenTransition(T newRead, T newWrite, typename std::vector<state<T> >::size_type newNextState, typename std::vector<state<T> >::size_type newOriginalState, typename infinite_tape<T>::direction newMove)
	:read(newRead), write(newWrite), originalState(newOriginalState), nextState(newNextState), move(newMove)
	{}

	T read;
	T write;
	typename std::vector<state<T> >::size_type originalState;
	typename std::vector<state<T> >::size_type nextState;
	typename infinite_tape<T>::direction move;
};

/**
 * represents a one or two cell portion of a turing machine tape.
 * @ingroup NonHaltDetectionDataStructures
 */
template <typename T>
class localTape
{
public:
	/**
	 * default constructor.
	 * @author Owen Kellett
	 */
	localTape()
	{}

	/**
	 * useful constructor.
	 * @pre T is a valid symbol on the input tape
	 * @post this is a one cell portion of tape which contains T as its value
	 * @param onlySymbol symbol to contruct one cell tape
	 * @author Owen Kellett
	 */
	localTape(T onlySymbol)
	{
		tapeConfig.push_back(onlySymbol);
		currentPosition = 0;
	}

	/**
	 * mechanism to facilitate local tape creation for back tracking algorithm
	 * @pre desiredSymbol is a valid symbol on the tape, transition is a valid transition
	 * @post creates a local tape that will accept the given transition and when the transition is executed on it, the read head will contain desiredSymbol
	 * @param desiredSymbol desired symbol for the read head after the transition is executed on the tape
	 * @param transition transition to be executed on the tape
	 * @author Owen Kellett
	 */
	localTape(T desiredSymbol, owenTransition<T> &transition)
	{
		switch(transition.move)
		{
			case infinite_tape<T>::left:
				tapeConfig.push_back(desiredSymbol);
				tapeConfig.push_back(transition.read);
				currentPosition = 1;
				break;
			case infinite_tape<T>::right:
				tapeConfig.push_back(transition.read);
				tapeConfig.push_back(desiredSymbol);
				currentPosition = 0;
				break;
			case infinite_tape<T>::do_nothing:
				tapeConfig.push_back(transition.read);
				currentPosition = 0;
				break;
		}
	}

	/**
	 * creates tape that will result in desiredTape after transition is performed
	 * @pre desiredTape is a valid tape and transition is a valid transition
	 * @post creates a local tape that will accept the given transition and when the transition is executed on it, the tape will be the desiredTape
	 * @param desiredTape desired tape after transition
	 * @param transition transition to be executed
	 * @author Owen Kellett
	 */
	localTape(localTape<T> &desiredTape, owenTransition<T> &transition)
	{
		switch(transition.move)
		{
			case infinite_tape<T>::left:
				for(int j = 0; j <= desiredTape.tapeConfig.size(); j++)
				{
					if(j == desiredTape.currentPosition + 1)
					{
						tapeConfig.push_back(transition.read);
						currentPosition = tapeConfig.size()-1;
					}
					else if(j == desiredTape.tapeConfig.size())
						continue;
					else
						tapeConfig.push_back(desiredTape.tapeConfig[j]);
				}
				break;
			case infinite_tape<T>::right:
				if(desiredTape.currentPosition == 0)
				{
					tapeConfig.push_back(transition.read);
					currentPosition = tapeConfig.size()-1;
				}
				for(int j = 0; j < desiredTape.tapeConfig.size(); j++)
				{
					if(j == desiredTape.currentPosition - 1)
					{
						tapeConfig.push_back(transition.read);
						currentPosition = tapeConfig.size()-1;
					}
					else
						tapeConfig.push_back(desiredTape.tapeConfig[j]);
				}
				break;
			case infinite_tape<T>::do_nothing:
				for(int j = 0; j < desiredTape.tapeConfig.size(); j++)
				{
					if(j == desiredTape.currentPosition)
					{
						tapeConfig.push_back(transition.read);
						currentPosition = tapeConfig.size()-1;
					}
					else
						tapeConfig.push_back(desiredTape.tapeConfig[j]);
				}
				break;
		}
	}

	/**
	 * simulates a given transition on "this" local tape
	 * @pre desiredTape is a valid representation of a localTape, transition is a valid transition
	 * @post when transition is executed on "this" local tape, if the conceived new form of this is functionally equivalent to the desired tape, return true else false
	 * @param desiredTape desired tape after transition
	 * @param transition transition to be executed
	 * @author Owen Kellett
	 */
	bool simulateTransition(localTape<T> &desiredTape, owenTransition<T> &transition)
	{
		std::vector<T> tempTapeConfig(tapeConfig);
		int tempCurrentPosition = currentPosition;
		int tempCounter, desiredCounter;

		if(tempTapeConfig[tempCurrentPosition] == transition.read)
		{
			//simulate the transition on a temporary copy of this tape
			tempTapeConfig[tempCurrentPosition] = transition.write;
			switch(transition.move)
			{
				case infinite_tape<T>::left:
					tempCurrentPosition--;
					break;
				case infinite_tape<T>::right:
					tempCurrentPosition++;
					break;
			}

			//check if the tapes are equivalent to the right of the read head
			for(tempCounter = tempCurrentPosition, desiredCounter = desiredTape.currentPosition;
				tempCounter < tempTapeConfig.size() && desiredCounter < desiredTape.tapeConfig.size();
				tempCounter++, desiredCounter++)
			{
				if(tempTapeConfig[tempCounter] != desiredTape.tapeConfig[desiredCounter])
				{
					return false;
				}
			}
			//check if the tapes are equivalent to the left of the read head
			for(tempCounter = tempCurrentPosition, desiredCounter = desiredTape.currentPosition;
				tempCounter >= 0 && desiredCounter >= 0;
				tempCounter--, desiredCounter--)
			{
				if(tempTapeConfig[tempCounter] != desiredTape.tapeConfig[desiredCounter])
				{
					return false;
				}
			}
			return true;
		}
		else
			return false;
	}

	/**
	 * see if the tape is all one value
	 * @post true if every cell on the tape is equal to value
	 * @author Owen Kellett
	 */
	bool allOneValue(T value)
	{
		for(int j = 0; j < tapeConfig.size(); j++)
			if(tapeConfig[j] != value)
				return false;
		return true;
	}

	/**
	 * prints the tape.
	 * used mainly for debugging purposes
	 * @author Owen Kellett
	 */
	void print()
	{
		for(int j = 0; j < tapeConfig.size(); j++)
		{
			if(j == currentPosition)
				std::cout << "->" << tapeConfig[j] << "<-";
			else
				std::cout << tapeConfig[j];
		}
	}

	std::vector<T> tapeConfig;
	int currentPosition;
};

//end backTrack data structures

//begin christmasTree data structures

/**
 * representation of an arbitrary size local tape configuration.
 * @ingroup NonHaltDetectionDataStructures
 */
template <typename T>
class tapeChunk
{
public:
	/**
	 * default constructor.
	 * @author Owen Kellett
	 */
	tapeChunk()
	{}

	/**
	 * copy constructor.
	 * @param tape tape chunk to be copied
	 * @author Owen Kellett
	 */
	tapeChunk(const tapeChunk<T> &tape)
	{
		for(int j = 0; j < tape.tapeConfig.size(); j++)
			tapeConfig.push_back(tape.tapeConfig[j]);
	}

	/**
	 * creates a local tape representing the cells from start to end on tape.
	 * @pre tape is a valid infinite_tape and start and end point to cells on tape
	 * @post *this is a local tape representing the cells from start to end on tape
	 * @param tape the tape to derive the local tape configuration from
	 * @param start the left edge of the tape chunk
	 * @param end the right edge of the tape chunk
	 * @author Owen Kellett
	 */
	tapeChunk(infinite_tape<T> &tape, typename std::list<T>::iterator start, typename std::list<T>::iterator end)
	{
		for(typename std::list<T>::iterator index = start; index != end; index++)
			tapeConfig.push_back(*index);
		tapeConfig.push_back(*end);
	}

	/**
	 * creates a local tape representing the cells from start to end on tape.
	 * @pre tape is a valid tapeChunk and start and end are locations within the range of tape's limits
	 * @post *this is a local tape representing the cells from start to end on tape
	 * @param tape the tape to derive the local tape configuration from
	 * @param start the left edge of the tape chunk
	 * @param end the right edge of the tape chunk
	 * @author Owen Kellett
	 */
	tapeChunk(tapeChunk<T> &tape, int start, int end)
	{
		for(int j = start; j < end; j++)
			tapeConfig.push_back(tape.tapeConfig[j]);
		tapeConfig.push_back(tape.tapeConfig[end]);
	}

	/**
	 * fills the tape with the local tape configuration on tape from start to end
	 * @pre tape is a valid infinite_tape and start and end point to cells on tape
	 * @post *this is a local tape representing the cells from start to end on tape
	 * @param tape the tape to derive the local tape configuration from
	 * @param start the left edge of the tape chunk
	 * @param end the right edge of the tape chunk
	 * @author Owen Kellett
	 */
	void fillTape(infinite_tape<T> &tape, typename std::list<T>::iterator start, typename std::list<T>::iterator end)
	{
		for(typename std::list<T>::iterator index = start; index != end; index++)
					tapeConfig.push_back(*index);
		tapeConfig.push_back(*end);
	}

	/**
	 * fills the tape with the local tape configuration representing the cells from start to end on tape.
	 * @pre tape is a valid tapeChunk and start and end are locations within the range of tape's limits
	 * @post *this is a local tape representing the cells from start to end on tape
	 * @param tape the tape to derive the local tape configuration from
	 * @param start the left edge of the tape chunk
	 * @param end the right edge of the tape chunk
	 * @author Owen Kellett
	 */
	void fillTape(const tapeChunk<T> &tape, int start, int end)
	{
		for(int j = start; j <= end; j++)
			tapeConfig.push_back(tape.tapeConfig[j]);
		//tapeConfig.push_back(tape.tapeConfig[end]);
	}

	/**
	 * empties the tape.
	 * @post *this is a zero length tapeChunk
	 * @author Owen Kellett
	 */
	void emptyTape()
	{
		while(tapeConfig.size() > 0)
			tapeConfig.pop_back();
	}

	/**
	 * compare the elements in tape with the elements in *this starting with cell start
	 * @return if the every element on tape matches the corresponding sequence of elements on *this starting with cell start, return true else false
	 * @author Owen Kellett
	 */
	bool compareNextChunk(const tapeChunk<T> & tape, int &start)
	{
		for(int j = 0; j < tape.tapeConfig.size(); j++)
		{
			if(start >= tapeConfig.size() || tape.tapeConfig[j] != tapeConfig[start])
				return false;
			start++;
		}
		return true;
	}

	/**
	 * is the tape empty?
	 * @return true iff the tape is empty
	 * @author Owen Kellett
	 */
	bool isEmpty()
	{
		return tapeConfig.size() == 0;
	}

	/**
	 * length of the tape.
	 * @return number of cells in *this
	 * @author Owen Kellett
	 */
	int length()
	{
		return tapeConfig.size();
	}

	/**
	 * prints the tape.
	 * @author Owen Kellett
	 */
	void print()
	{
		for(int j = 0; j < tapeConfig.size(); j++)
			std::cout << tapeConfig[j] << ' ';
		std::cout << std::endl;
	}

	/**
	 * concatenates two tapeChunks together
	 * @author Owen Kellett
	 */
	tapeChunk<T> operator + (const tapeChunk<T> &rhs)
	{
		tapeChunk<T> returnValue;
		returnValue.fillTape(*this, 0, tapeConfig.size()-1);
		returnValue.fillTape(rhs, 0, rhs.tapeConfig.size()-1);
		return returnValue;
	}

	/**
	 * equality operator
	 * @return true iff tapes are identical
	 * @author Owen Kellett
	 */
	bool operator == (const tapeChunk<T> &rhs)
	{
		if(tapeConfig.size() != rhs.tapeConfig.size())
			return false;
		for(int j = 0; j < tapeConfig.size(); j++)
			if(tapeConfig[j] != rhs.tapeConfig[j])
				return false;
		return true;
	}

	/**
	 * inequality operator
	 * @return true iff tapes are not identical
	 * @author Owen Kellett
	 */
	bool operator != (const tapeChunk<T> &rhs)
	{
		if(tapeConfig.size() != rhs.tapeConfig.size())
			return true;
		for(int j = 0; j < tapeConfig.size(); j++)
			if(tapeConfig[j] != rhs.tapeConfig[j])
				return true;
		return false;
	}

	std::vector<T> tapeConfig;
};



//end christmasTree data structures

//being multisweep data structures

/**
 * holds data for extracted components of multi-sweep christmas trees.
 * @ingroup NonHaltDetectionDataStructures
 */
template <typename T>
class sweepStats
{
public:
	tapeChunk<T> U;
	tapeChunk<T> V;
	tapeChunk<T> X;

	tapeChunk<T> Xp;
	tapeChunk<T> Y;
	tapeChunk<T> Yp;
	tapeChunk<T> Z;
	tapeChunk<T> Vp;
	tapeChunk<T> Vpp;
	tapeChunk<T> Up;

	typename std::vector<state<T> >::size_type q;
	typename std::vector<state<T> >::size_type r;
	typename std::vector<state<T> >::size_type t;

	void fill(tapeChunk<T> myU,
			tapeChunk<T> myV,
			tapeChunk<T> myX,
			tapeChunk<T> myXp,
			tapeChunk<T> myY,
			tapeChunk<T> myYp,
			tapeChunk<T> myZ,
			tapeChunk<T> myVp,
			tapeChunk<T> myVpp,
			tapeChunk<T> myUp,
			typename std::vector<state<T> >::size_type myq,
			typename std::vector<state<T> >::size_type myr,
			typename std::vector<state<T> >::size_type myt)
	{
		U = myU;
		V = myV;
		X = myX;
		Xp = myXp;
		Y = myY;
		Yp = myYp;
		Z = myZ;
		Vp = myVp;
		Vpp = myVpp;
		Up = myUp;
		q = myq;
		r = myr;
		t = myt;
	}

	void populate(tapeChunk<T> &myU,
			tapeChunk<T> &myV,
			tapeChunk<T> &myX,
			tapeChunk<T> &myXp,
			tapeChunk<T> &myY,
			tapeChunk<T> &myYp,
			tapeChunk<T> &myZ,
			tapeChunk<T> &myVp,
			tapeChunk<T> &myVpp,
			tapeChunk<T> &myUp,
			typename std::vector<state<T> >::size_type &myq,
			typename std::vector<state<T> >::size_type &myr,
			typename std::vector<state<T> >::size_type &myt)
	{
		myU = U;
		myV = V;
		myX = X;
		myXp = Xp;
		myY = Y;
		myYp = Yp;
		myZ = Z;
		myVp = Vp;
		myVpp = Vpp;
		myUp = Up;
		myq = q;
		myr = r;
		myt = t;
	}
};

//end multisweep data structures

#endif
