#ifndef kellett_non_halt_detection_header
#define kellett_non_halt_detection_header

/*Authored 2003    by  Owen Kellett,  UnderGraduate Research
(Advisor: Professor Bram van Heuveln)


This  file is part of the   "Generic Turing Machine Implementation for
Use in the Exploration of the Busy Beaver Problem".

The "Generic Turing Machine Implementation for  Use in the Exploration
of the Busy Beaver Problem" is free  software; you can redistribute it
and/or modify it under the terms of the  GNU General Public License as
published   by the Free Software Foundation;   either version 2 of the
License, or (at your option) any later version.

This programme is distributed in the hope that it  will be useful, but
WITHOUT    ANY  WARRANTY;  without   even  the    implied warranty  of
MERCHANTABILITY or FITNESS FOR  A  PARTICULAR  PURPOSE.  See  the  GNU
General Public License for more details.

You should have received   a copy of  the  GNU General Public  License
along  with  this   programme; if  not,  write   to the Free  Software
Foundation, Inc., 59  Temple Place, Suite  330, Boston, MA  02111-1307
USA (or visit http://www.fsf.org/licenses/licenses.html)

Created as part of the Busy Beaver Project:
Bram van Heuveln <heuveb@rpi.edu>
Boleshaw Szymanski <szymansk@cs.rpi.edu>
Selmer Bringsjord <selmer@rpi.edu>
Carlos Varela <cvarela@cs.rpi.edu>
Owen Kellett <kelleo@rpi.edu>
Shailesh Kelkar <kelkas@rpi.edu>
Kyle Ross <rossk2@rpi.edu>
Rensselaer Reasoning Group
Department of Cognitive Science
Department of Computer Science
Rensselaer Polytechnic Institute
Troy, New York 12180
U.S.A.
*/

#include <fstream>
#include <iostream>
#include <iterator>
#include <list>
#include <vector>
#include <map>

#include "infinite_tape.hpp"
#include "state.hpp"
#include "turing_machine.hpp"
#include "non_halt_data_structures.hpp"

typedef unsigned short d_type;

/**
 * @defgroup NonHaltDetectionRoutines
 */

/**
 * Simple loop detection algorithm
 * @pre machine is a valid turing_machine
 * @post true if machine is determined to be a non halter according to the simple loop behavior with a step bound of transition_limit
 * @param machine turing machine to check
 * @transition_limit arbitrary step limit at which the check fails
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
bool simpleLoopCheck(turing_machine<d_type> &machine, unsigned int transition_limit)
{
	machine.resetMachine();

	std::map<stateSymbolPair<d_type>, std::vector<tapeBoundsTriple<d_type> > > loopChecksMap;

	unsigned int step_counter = 0;
	while(step_counter < transition_limit && machine.run_one()!=turing_machine<d_type>::halt && !machine.blank_tape())
	{
		++step_counter;
		std::map<stateSymbolPair<d_type>, std::vector<tapeBoundsTriple<d_type> > >::iterator checkBounds = loopChecksMap.begin();
		for(; checkBounds != loopChecksMap.end(); checkBounds++)
		{
			for(int i = 0; i < (*checkBounds).second.size(); i++)
			{
				(*checkBounds).second[i].tape.checkIteratorLeft((*checkBounds).second[i].leftBound);
				(*checkBounds).second[i].tape.checkIteratorRight((*checkBounds).second[i].rightBound);
				machine.access_tape_read_write().checkIteratorLeft((*checkBounds).second[i].leftBoundOriginal);
				machine.access_tape_read_write().checkIteratorRight((*checkBounds).second[i].rightBoundOriginal);

				if(machine.access_tape_read_write().currentLocation() != --((*checkBounds).second[i].leftBoundOriginal))
					(*checkBounds).second[i].leftBoundOriginal++;
				else
					(*checkBounds).second[i].leftBound--;
				if(machine.access_tape_read_write().currentLocation() != ++((*checkBounds).second[i].rightBoundOriginal))
					(*checkBounds).second[i].rightBoundOriginal--;
				else
					(*checkBounds).second[i].rightBound++;
			}
		}

		stateSymbolPair<d_type> tempPair(machine.get_current_state(), machine.access_tape().read());

		int size = loopChecksMap[tempPair].size();
		for(int i = 0; i < size; i++)
		{
			if(loopChecksMap[tempPair][i].tape.repeatConfigRight(machine.access_tape_read_write(), loopChecksMap[tempPair][i].leftBound))
			{
				return true;
			}
			if(loopChecksMap[tempPair][i].tape.repeatConfigLeft(machine.access_tape_read_write(), loopChecksMap[tempPair][i].rightBound))
			{
				return true;
			}
		}
		tapeBoundsTriple<d_type> newTapeTriple(machine.access_tape_read_write(),
							machine.access_tape_read_write().currentLocation(),
							machine.access_tape_read_write().currentLocation());
		loopChecksMap[tempPair].push_back(newTapeTriple);
	}
	return false;

}

/**
 * recursive back track helper.
 * @pre stateSet represents a set of turing machine states (which includes a map of their individual transitions); tape and stateIndex is the "current state of the machine"
 * @post through backtracking, true if it is impossible to reach the "current state of the machine"
 * @param stateSet set of states in the turing machine
 * @param tape tape required to satisfy back track step
 * @param stateIndex state required to satsify back track step
 * @param limitSeed current back track step
 * @param backTrackLimit max back track steps before proof fails by default
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
bool isInfinite(std::vector<state<d_type> > &stateSet, localTape<d_type>& tape, std::vector<state<d_type> >::size_type stateIndex, int limitSeed, int backTrackLimit)
{
	if(limitSeed == backTrackLimit || (stateIndex == 0 && tape.allOneValue(0)))
	{
		/*std::cout << limitSeed << std::endl;
		std::cout << stateIndex << std::endl;
		tape.print(); std::cout << std::endl;*/
		return false;
	}

	d_type write;
	infinite_tape<d_type>::direction move;
	std::vector<state<d_type> >::size_type nextState;

	std::vector<owenTransition<d_type> > possibleTransitions;
	std::vector<localTape<d_type> > requiredTapes;

	/*std::cout << "Depth: " << limitSeed << std::endl;
	tape.print(); std::cout << std::endl;
	std::cout << stateIndex << std::endl << std::endl;*/

	for(int j = 0; j < stateSet.size(); j++)
	{
		if(stateSet[j].simulate_transition(0, write, move, nextState) == state<d_type>::run && nextState == stateIndex)
			possibleTransitions.push_back(owenTransition<d_type>(0, write, nextState, j, move));
		if(stateSet[j].simulate_transition(1, write, move, nextState) == state<d_type>::run && nextState == stateIndex)
			possibleTransitions.push_back(owenTransition<d_type>(1, write, nextState, j, move));
	}
	for(int j = 0; j < possibleTransitions.size(); j++)
	{
		//requiredTapes.push_back(localTape<d_type>(tape.tapeConfig[tape.currentPosition], possibleTransitions[j]));
		requiredTapes.push_back(localTape<d_type>(tape, possibleTransitions[j]));
		if(requiredTapes[j].simulateTransition(tape, possibleTransitions[j]))
		{
			if(!isInfinite(stateSet, requiredTapes[j], possibleTransitions[j].originalState, limitSeed+1, backTrackLimit))
				return false;
		}
	}
	return true;
}

/**
 * parent backTrack function.
 * @post true if machine is determined to be a non-halter through backTracking methods
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
bool backTrack(turing_machine<d_type> &machine, unsigned int transition_limit)
{
	std::vector<state<d_type> > stateSet = machine.stateSet();

	d_type write;
	infinite_tape<d_type>::direction move;
	std::vector<state<d_type> >::size_type nextState;
	std::vector<owenTransition<d_type> > possibleTransitions;

	for(int j = 0; j < stateSet.size(); j++)
	{
		if(stateSet[j].simulate_transition(0, write, move, nextState) == state<d_type>::run)
		{
			if(nextState == machine.haltState())
				possibleTransitions.push_back(owenTransition<d_type>(0, write, nextState, j, move));
		}
		else
		{
			possibleTransitions.push_back(owenTransition<d_type>(0, 0, machine.haltState(), j, infinite_tape<d_type>::do_nothing));
		}
		if(stateSet[j].simulate_transition(1, write, move, nextState) == state<d_type>::run)
		{
			if(nextState == machine.haltState())
				possibleTransitions.push_back(owenTransition<d_type>(1, write, nextState, j, move));
		}
		else
		{
			possibleTransitions.push_back(owenTransition<d_type>(1, 1, machine.haltState(), j, infinite_tape<d_type>::do_nothing));
		}
	}
	for(int j = 0; j < possibleTransitions.size(); j++)
	{
		localTape<d_type> requiredTape(possibleTransitions[j].read);
		if(!isInfinite(stateSet, requiredTape, possibleTransitions[j].originalState, 0, transition_limit))
			return false;
	}
	return true;
}

/**
 * check if element is in set
 * @param element state to check
 * @param set set of states to search in
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
bool isIn(std::vector<state<d_type> >::size_type element,
	  std::vector<std::vector<state<d_type> >::size_type> set)
{
	for(int j = 0; j < set.size(); j++)
		if(set[j] == element)
			return true;
	return false;
}

/**
 * subset loops helper function.
 * searches for an inescapable set of states given a starting state
 * @param seed current state of the machine
 * @param haltState halting state of the machine
 * @param stateSet set of all states in the machine
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
bool subsetLoop(std::vector<state<d_type> >::size_type seed,
		std::vector<state<d_type> >::size_type haltState,
		std::vector<state<d_type> > &stateSet)
{
	std::vector<std::vector<state<d_type> >::size_type> subset;

	d_type write;
	infinite_tape<d_type>::direction move;
	std::vector<state<d_type> >::size_type nextState;
	std::vector<std::vector<state<d_type> >::size_type>::size_type j;

	subset.push_back(seed);
	for(j = 0; j < subset.size(); j++)
	{
		if(stateSet[subset[j]].simulate_transition(0, write, move, nextState) == state<d_type>::halt || nextState == haltState)
			return false;
		else if(!isIn(nextState, subset))
			subset.push_back(nextState);

		if(stateSet[subset[j]].simulate_transition(1, write, move, nextState) == state<d_type>::halt || nextState == haltState)
                        return false;
                else if(!isIn(nextState, subset))
                        subset.push_back(nextState);
	}
	return true;
}

/**
 * subset loop non halt detection.
 * searches for an inescapable set of states in machine
 * @post true if machine is a subset loop non halter
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
bool loopStates(turing_machine<d_type> &machine)
{
        std::vector<state<d_type> > stateSet = machine.stateSet();
        std::vector<state<d_type> >::size_type j;

        for(j = 0; j < stateSet.size(); j++)
                if(subsetLoop(j, machine.haltState(), stateSet))
                        return true;

        return false;
}


//begin christmasTree routines

/**
 * get the next right extremum in the back and forth sweeping motion of a christmas tree.
 * @post rightBound and leftBound are updated to the new values as well as step_counter
 * @param machine machine to check
 * @param rightBound location of previous right bound
 * @param leftBound location of previous left bound
 * @param step_counter current step of the machine
 * @param transition_limit point at which to stop searching
 * @return the step counter of the point at which the machine is in the next right most extremum
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
unsigned int getNextRightBound(turing_machine<d_type> &machine,
							   std::list<d_type>::iterator &rightBound,
							   std::list<d_type>::iterator &leftBound,
							   unsigned int &step_counter,
							   unsigned int transition_limit)
{
	bool rightBoundPushed = false;
	unsigned int rightBoundStepCounter = 0;

	for(step_counter; step_counter < transition_limit; step_counter++)
	{
		if(machine.access_tape_read_write().currentLocation() == rightBound)
		{
			rightBoundPushed = true;
			rightBoundStepCounter = step_counter-1;
			if(machine.nextMove() ==  infinite_tape<d_type>::right)
			{
				machine.access_tape_read_write().checkIteratorRight(rightBound);
				rightBound++;
				rightBoundPushed = true;
				rightBoundStepCounter = step_counter;
			}
		}
		if(machine.access_tape_read_write().currentLocation() == leftBound)
		{
			if(machine.nextMove() ==  infinite_tape<d_type>::left)
			{
				machine.access_tape_read_write().checkIteratorLeft(leftBound);
				leftBound--;
				if(rightBoundPushed)
				{
					return rightBoundStepCounter;
				}
			}
			if(rightBoundPushed)
			{
				return rightBoundStepCounter;
			}
		}
		if(machine.run_one()==turing_machine<d_type>::halt)
			return 0;
	}
	return 0;
}

/**
 * get next left bound [not used].
 * @author Owen Kellett
 */
unsigned int getNextLeftBound(turing_machine<d_type> &machine,
							   std::list<d_type>::iterator &rightBound,
							   std::list<d_type>::iterator &leftBound,
							   unsigned int &step_counter,
							   unsigned int transition_limit)
{
	bool leftBoundPushed = false;
	unsigned int leftBoundStepCounter;

	for(step_counter; step_counter < transition_limit; step_counter++)
	{
		if(machine.access_tape_read_write().currentLocation() == rightBound)
			if(machine.nextMove() ==  infinite_tape<d_type>::right)
			{
				machine.access_tape_read_write().checkIteratorRight(rightBound);
				rightBound++;
				if(leftBoundPushed)
				{
					return leftBoundStepCounter;
				}
			}
		if(machine.access_tape_read_write().currentLocation() == leftBound)
			if(machine.nextMove() ==  infinite_tape<d_type>::left)
			{
				machine.access_tape_read_write().checkIteratorLeft(leftBound);
				leftBound--;
				leftBoundPushed = true;
				leftBoundStepCounter = step_counter;
			}
		if(machine.run_one()==turing_machine<d_type>::halt)
			return 0;
	}
	return 0;
}

/**
 * finds an additional middle portion of the tape given a tape before and after a sweep of a christmas tree.
 * @pre before and after have common end components; before has no additional componenets while after has an arbitrary length additional center component;
 * @post midSection, leftSection, and rightSection are filled with the left, middle, and right components of after (left and right are also the same as the left and right components in before)
 * @param before representation of tape before sweep of the tree
 * @param after representation of tape after sweep of the tree
 * @param midSection reference to tapeChunk to populate with middle component
 * @param leftSection reference to tapeChunk to poulate with left component
 * @param rightSection reference to tapeChunk to populate with right component
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
bool findMidSection(tapeChunk<d_type> &before, tapeChunk<d_type> &after, tapeChunk<d_type> &midSection, tapeChunk<d_type> &leftSection, tapeChunk<d_type> &rightSection)
{
	int j, k;
	for(j = 0; j < before.tapeConfig.size()/2; j++)
	{
		if(before.tapeConfig[j] != after.tapeConfig[j])
			return false;
	}
	for(j = before.tapeConfig.size()-1, k = after.tapeConfig.size()-1; j >= before.tapeConfig.size()/2; j--, k--)
	{
		if(before.tapeConfig[j] != after.tapeConfig[k])
			return false;
	}
	midSection.fillTape(after, before.tapeConfig.size()/2, k);
	leftSection.fillTape(before, 0, before.tapeConfig.size()/2 - 1);
	rightSection.fillTape(before, before.tapeConfig.size()/2, before.tapeConfig.size()-1);
	return true;
}

/**
 * get the next chunk of the tape after performing a transformation in the christmas tree grammars.
 * runs the machine according to the following conditions.
 * if stopAtLeftBound runs the machine until the read head hits leftBound.
 * if stopAtRightBoudn runs the machine until the read head hits right bound.
 * if neither runs the machine until the specified stopPointCounter.
 * returns the tapeChunk representative of the tape between leftBound and rightBound after the transformation.
 * returns blank tape if left or right bounds are breached when they shouldn't be
 * @param leftBound left boundary of the transformation
 * @param rightBound right boundary of the transformation
 * @param stopPointCounter counter to stop at if specified to
 * @param step_counter current step counter of machine
 * @param transition_limit max limit to go to
 * @param stopAtLeftBound signals to run until leftBound is hit
 * @param stopAtRightBound signals to run until rightBound is hit
 * @param inclusiveLeft signals whether or not to include the left most symbol in the answer after execution
 * @param inclusiveRight signals whether or not to include the right most symbol in the answer after execution
 * @param pushLeftBound signals to push the leftBound on the tape if it is hit (during transitions at left of sweep)
 * @param pushRightBound signals to put the rightBound on the tape if it is hit (during transitions at right of sweep)
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
tapeChunk<d_type> getNextChunk(turing_machine<d_type> &machine,
							   std::list<d_type>::iterator leftBound,
							   std::list<d_type>::iterator rightBound,
							   unsigned int stopPointCounter,
							   unsigned int &step_counter,
							   unsigned int transition_limit,
							   bool stopAtLeftBound,
							   bool stopAtRightBound,
							   bool inclusiveLeft,
							   bool inclusiveRight,
							   bool pushLeftBound,
							   bool pushRightBound)
{
	if(stopAtLeftBound)
	{
		while(machine.access_tape_read_write().currentLocation() != leftBound)
		{
			if(machine.access_tape_read_write().currentLocation() == rightBound)
			{
				if(machine.nextMove() ==  infinite_tape<d_type>::right)
				{
					if(pushRightBound)
					{
						machine.access_tape_read_write().checkIteratorRight(rightBound);
						rightBound++;
					}
					else
						return tapeChunk<d_type>();
				}
			}
			machine.run_one();
			step_counter++;
			if(step_counter == transition_limit)
				return tapeChunk<d_type>();
		}
	}
	else if(stopAtRightBound)
	{
		while(machine.access_tape_read_write().currentLocation() != rightBound)
		{
			if(machine.access_tape_read_write().currentLocation() == leftBound)
			{
				if(machine.nextMove() ==  infinite_tape<d_type>::left)
				{
					if(pushLeftBound)
					{
						machine.access_tape_read_write().checkIteratorLeft(leftBound);
						leftBound--;
					}
					else
						return tapeChunk<d_type>();
				}
			}
			machine.run_one();
			step_counter++;
			if(step_counter == transition_limit)
				return tapeChunk<d_type>();
		}
	}
	else
	{
		for(step_counter; step_counter <= stopPointCounter; step_counter++)
		{
			if(machine.access_tape_read_write().currentLocation() == rightBound)
			{
				if(machine.nextMove() ==  infinite_tape<d_type>::right)
				{
					if(pushRightBound)
					{
						machine.access_tape_read_write().checkIteratorRight(rightBound);
						rightBound++;
					}
					else
						return tapeChunk<d_type>();
				}
			}
			if(machine.access_tape_read_write().currentLocation() == leftBound)
			{
				if(machine.nextMove() ==  infinite_tape<d_type>::left)
				{
					if(pushLeftBound)
					{
						machine.access_tape_read_write().checkIteratorLeft(leftBound);
						leftBound--;
					}
					else
						return tapeChunk<d_type>();
				}
			}
			machine.run_one();
			if(step_counter == transition_limit)
				return tapeChunk<d_type>();
		}
	}

	if(!inclusiveLeft)
		leftBound++;
	if(!inclusiveRight)
		rightBound--;
	tapeChunk<d_type> returnValue;
	returnValue.fillTape(machine.access_tape_read_write(), leftBound, rightBound);

	return returnValue;
}

bool checkChristmasTreeGrammar(turing_machine<d_type> &machine,
							   unsigned int firstRightBound,
							   unsigned int secondRightBound,
							   unsigned int thirdRightBound,
							   unsigned int transition_limit,
							   tapeChunk<d_type> &U,
							   tapeChunk<d_type> &X,
							   tapeChunk<d_type> &V,
							   std::vector<state<d_type> >::size_type s)
{
	machine.resetMachine();
	unsigned int step_counter = 0;
	tapeChunk<d_type> Xp, Y, Yp, Z, Vp, Vpp, Up, temp;
	std::vector<state<d_type> >::size_type q, r;

	for(step_counter; step_counter <= firstRightBound; step_counter++)
		machine.run_one();

	std::list<d_type>::iterator leftBound, rightBound;

	//XVs0* -> qX'V'0*
	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int j = 0; j < V.tapeConfig.size() + X.tapeConfig.size(); j++)
		leftBound--;
	temp = getNextChunk(machine,
						leftBound,
						rightBound,
						secondRightBound,
						step_counter,
						transition_limit,
						true,				//stop at left
						false,				//stop at right
						false,				//inclusive left
						true,				//inclusive right
						false,				//push left
						false);				//push right
	if(temp.isEmpty()) return false;
	q = machine.get_current_state();
	Xp.fillTape(temp, 0, X.tapeConfig.size()-1);
	Vp.fillTape(temp, X.tapeConfig.size(), temp.tapeConfig.size()-1);

	//0*UqX' -> 0*U'Y'r
	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int j = 0; j < Xp.tapeConfig.size()+1; j++)
		rightBound++;
	temp = getNextChunk(machine,
						leftBound,
						rightBound,
						secondRightBound,
						step_counter,
						transition_limit,
						false,				//stop at left
						true,				//stop at right
						true,				//inclusive left
						false,				//inclusive right
						true,				//push left
						false);				//push right
	if(temp.isEmpty()) return false;
	r = machine.get_current_state();
	Up.fillTape(temp, 0, temp.tapeConfig.size()-X.tapeConfig.size()-1);
	Yp.fillTape(temp, temp.tapeConfig.size()-X.tapeConfig.size(), temp.tapeConfig.size()-1);

	//Y'rV' -> ZVs''
	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int j = 0; j < Yp.tapeConfig.size(); j++)
		leftBound--;
	temp = getNextChunk(machine,
						leftBound,
						rightBound,
						secondRightBound,
						step_counter,
						transition_limit,
						false,				//stop at left
						false,				//stop at right
						true,				//inclusive left
						true,				//inclusive right
						false,				//push left
						true);				//push right
	if(temp.isEmpty()) return false;
	Z.fillTape(temp, 0, X.tapeConfig.size()-1);
	Vpp.fillTape(temp, X.tapeConfig.size(), temp.tapeConfig.size()-1);
	if(machine.get_current_state() != s)
		return false;

	/*std::cout << "Up : "; Up.print();
	std::cout << "Z  : "; Z.print();
	std::cout << "Vpp: "; Vpp.print();
	std::cout << "U :  "; U.print();
	std::cout << "X :  "; X.print();
	std::cout << "V :  "; V.print();*/

	if(Up + Z + Vpp != U + X + X + V)
		return false;

	//XVs0* -> qX'V'0*
	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int j = 0; j < V.tapeConfig.size() + X.tapeConfig.size(); j++)
		leftBound--;
	temp = getNextChunk(machine,
						leftBound,
						rightBound,
						secondRightBound,
						step_counter,
						transition_limit,
						true,				//stop at left
						false,				//stop at right
						false,				//inclusive left
						true,				//inclusive right
						false,				//push left
						false);				//push right
	if(temp.isEmpty()) return false;
	if(temp != Xp + Vp || machine.get_current_state() != q)
		return false;

	//XqX' -> qX'Y
	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int j = 0; j < X.tapeConfig.size(); j++)
		leftBound--;
	for(int j = 0; j < Xp.tapeConfig.size(); j++)
		rightBound++;
	temp = getNextChunk(machine,
						leftBound,
						rightBound,
						secondRightBound,
						step_counter,
						transition_limit,
						true,				//stop at left
						false,				//stop at right
						false,				//inclusive left
						true,				//inclusive right
						false,				//push left
						false);				//push right
	if(temp.isEmpty()) return false;
	Y.fillTape(temp, Xp.tapeConfig.size(), temp.tapeConfig.size()-1);
	if(temp != Xp+Y || machine.get_current_state() != q)
		return false;

	//0*UqX' -> 0*U'Y'r
	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int j = 0; j < Xp.tapeConfig.size()+1; j++)
		rightBound++;
	temp = getNextChunk(machine,
						leftBound,
						rightBound,
						secondRightBound,
						step_counter,
						transition_limit,
						false,				//stop at left
						true,				//stop at right
						true,				//inclusive left
						false,				//inclusive right
						true,				//push left
						false);				//push right
	if(temp.isEmpty()) return false;
	if(temp != Up+Yp || machine.get_current_state() != r)
		return false;

	//Y'rY -> ZY'r
	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int j = 0; j < Yp.tapeConfig.size(); j++)
		leftBound--;
	for(int j = 0; j < Y.tapeConfig.size(); j++)
		rightBound++;
	temp = getNextChunk(machine,
						leftBound,
						rightBound,
						secondRightBound,
						step_counter,
						transition_limit,
						false,				//stop at left
						true,				//stop at right
						true,				//inclusive left
						false,				//inclusive right
						false,				//push left
						false);				//push right
	if(temp.isEmpty()) return false;
	if(temp != Z+Yp || machine.get_current_state() != r)
		return false;

	//Y'rV' -> ZVs''
	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int j = 0; j < Yp.tapeConfig.size(); j++)
		leftBound--;
	temp = getNextChunk(machine,
						leftBound,
						rightBound,
						thirdRightBound,
						step_counter,
						transition_limit,
						false,				//stop at left
						false,				//stop at right
						true,				//inclusive left
						true,				//inclusive right
						false,				//push left
						true);				//push right
	if(temp.isEmpty()) return false;
	if(temp != Z+Vpp || machine.get_current_state() != s)
		return false;
	if(Up + Z + Z + Vpp != U + X + X + X + V)
		return false;
	return true;
}

bool checkChristmasTree(turing_machine<d_type> &machine, unsigned int transition_limit)
{
	machine.resetMachine();
	unsigned int step_counter = 0;

	std::list<d_type>::iterator leftBound;
	std::list<d_type>::iterator rightBound;

	//last bound push
	//0 = left
	//1 = right
	int lastPush = 0;

	//run for a bit to establish sweeping motion
	for(step_counter = 0; step_counter < transition_limit &&
						  step_counter < 150; step_counter++)
	{
		if(step_counter == 50)
		{
			leftBound = machine.access_tape_read_write().currentLocation();
			rightBound = machine.access_tape_read_write().currentLocation();
		}
		if(step_counter >= 50)
		{
			if(machine.access_tape_read_write().currentLocation() == rightBound)
				if(machine.nextMove() ==  infinite_tape<d_type>::right)
				{
					machine.access_tape_read_write().checkIteratorRight(rightBound);
					rightBound++;
				}
			if(machine.access_tape_read_write().currentLocation() == leftBound)
				if(machine.nextMove() ==  infinite_tape<d_type>::left)
				{
					machine.access_tape_read_write().checkIteratorLeft(leftBound);
					leftBound--;
				}
		}
		if(machine.run_one()==turing_machine<d_type>::halt)
			return false;

	}

	//get the step counters of the left and right bounds
	unsigned int firstRightBound = getNextRightBound(machine, rightBound, leftBound, step_counter, transition_limit);
	if(firstRightBound == 0)
		return false;

	unsigned int secondRightBound = getNextRightBound(machine, rightBound, leftBound, step_counter, transition_limit);
	if(secondRightBound == 0)
		return false;

	unsigned int thirdRightBound = getNextRightBound(machine, rightBound, leftBound, step_counter, transition_limit);
	if(thirdRightBound == 0)
		return false;

	unsigned int fourthRightBound = getNextRightBound(machine, rightBound, leftBound, step_counter, transition_limit);
	if(fourthRightBound == 0)
		return false;

	//reset the machine and use step counters to get tape configuration at each point
	machine.resetMachine();
	tapeChunk<d_type> beforeSweep, afterSweep;

	std::vector<state<d_type> >::size_type stateS;

	for(unsigned int j = 0; j <= secondRightBound; j++)
	{
		if(j < 50)
			machine.run_one();
		if(j == 50)
			leftBound = machine.access_tape_read_write().currentLocation();
		if(j >= 50)
		{
			if(machine.access_tape_read_write().currentLocation() == leftBound)
				if(machine.nextMove() ==  infinite_tape<d_type>::left)
				{
					machine.access_tape_read_write().checkIteratorLeft(leftBound);
					leftBound--;
				}
			machine.run_one();

			if(j == firstRightBound)
			{
				rightBound = machine.access_tape_read_write().currentLocation();
				beforeSweep.fillTape(machine.access_tape_read_write(), leftBound, rightBound);
				stateS = machine.get_current_state();
			}

			if(j == secondRightBound)
			{
				rightBound = machine.access_tape_read_write().currentLocation();
				afterSweep.fillTape(machine.access_tape_read_write(), leftBound, rightBound);
				if(machine.get_current_state() != stateS)
					return false;
			}
		}
	}


	tapeChunk<d_type> chunkX, chunkU, chunkV;

	if(!findMidSection(beforeSweep, afterSweep, chunkX, chunkU, chunkV))
		return false;

	/*chunkX.print();
	chunkU.print();
	chunkV.print();*/

	if(!checkChristmasTreeGrammar(machine, secondRightBound, thirdRightBound, fourthRightBound, transition_limit, chunkU, chunkX, chunkV, stateS))
		return false;



	return true;
}

bool christmasTree(turing_machine<d_type> &machine, unsigned int transition_limit)
{
	if(checkChristmasTree(machine, transition_limit))
		return true;
	else
	{
		turing_machine<d_type> reverse = machine.reverseMachine();
		if(checkChristmasTree(reverse, transition_limit))
			return true;
	}
	return false;
}

//end christmasTree routines

//begin alternateChristmasTree routines

/**
 * specialized version of getNextChunk
 * this version is run for transitions that push the left bound in alternating christmas trees
 * picks up uneven alternating christmas tree.
 * get the next chunk of the tape after performing a transformation in the christmas tree grammars.
 * runs the machine according to the following conditions.
 * if stopAtLeftBound runs the machine until the read head hits leftBound.
 * if stopAtRightBoudn runs the machine until the read head hits right bound.
 * if neither runs the machine until the specified stopPointCounter.
 * returns the tapeChunk representative of the tape between leftBound and rightBound after the transformation.
 * returns blank tape if left or right bounds are breached when they shouldn't be
 * @param leftBound left boundary of the transformation
 * @param rightBound right boundary of the transformation
 * @param stopPointCounter counter to stop at if specified to
 * @param step_counter current step counter of machine
 * @param transition_limit max limit to go to
 * @param stopAtLeftBound signals to run until leftBound is hit
 * @param stopAtRightBound signals to run until rightBound is hit
 * @param inclusiveLeft signals whether or not to include the left most symbol in the answer after execution
 * @param inclusiveRight signals whether or not to include the right most symbol in the answer after execution
 * @param pushLeftBound signals to push the leftBound on the tape if it is hit (during transitions at left of sweep)
 * @param pushRightBound signals to put the rightBound on the tape if it is hit (during transitions at right of sweep)
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
tapeChunk<d_type> getNextChunkUnevenCheck(turing_machine<d_type> &machine,
							   std::list<d_type>::iterator leftBoundCheck,
							   std::list<d_type>::iterator leftBound,
							   std::list<d_type>::iterator rightBound,
							   unsigned int stopPointCounter,
							   unsigned int &step_counter,
							   unsigned int transition_limit,
							   bool stopAtLeftBound,
							   bool stopAtRightBound,
							   bool inclusiveLeft,
							   bool inclusiveRight,
							   bool pushLeftBound,
							   bool pushRightBound)
{
	if(stopAtLeftBound)
	{
		while(machine.access_tape_read_write().currentLocation() != leftBound)
		{
			if(pushRightBound)
				if(machine.access_tape_read_write().currentLocation() == rightBound)
					if(machine.nextMove() ==  infinite_tape<d_type>::right)
					{
						machine.access_tape_read_write().checkIteratorRight(rightBound);
						rightBound++;
					}
			machine.run_one();
			step_counter++;
			if(step_counter == transition_limit)
				return tapeChunk<d_type>();
		}
	}
	else if(stopAtRightBound)
	{
		while(machine.access_tape_read_write().currentLocation() != rightBound || leftBoundCheck != leftBound)
		{
			if(pushLeftBound)
				if(machine.access_tape_read_write().currentLocation() == leftBound)
					if(machine.nextMove() ==  infinite_tape<d_type>::left)
					{
						machine.access_tape_read_write().checkIteratorLeft(leftBound);
						leftBound--;
					}
			machine.run_one();
			step_counter++;
			if(step_counter == transition_limit)
				return tapeChunk<d_type>();
		}
	}
	else
	{
		for(step_counter; step_counter <= stopPointCounter; step_counter++)
		{
			if(pushRightBound)
				if(machine.access_tape_read_write().currentLocation() == rightBound)
					if(machine.nextMove() ==  infinite_tape<d_type>::right)
					{
						machine.access_tape_read_write().checkIteratorRight(rightBound);
						rightBound++;
					}
			if(pushLeftBound)
				if(machine.access_tape_read_write().currentLocation() == leftBound)
					if(machine.nextMove() ==  infinite_tape<d_type>::left)
					{
						machine.access_tape_read_write().checkIteratorLeft(leftBound);
						leftBound--;
					}
			machine.run_one();
			if(step_counter == transition_limit)
				return tapeChunk<d_type>();
		}
	}

	if(!inclusiveLeft)
		leftBound++;
	if(!inclusiveRight)
		rightBound--;
	tapeChunk<d_type> returnValue;
	returnValue.fillTape(machine.access_tape_read_write(), leftBound, rightBound);

	return returnValue;
}




//end alternateChristmasTree routines

//begin leaningChristmasTree routines

/**
 * check the grammar for a leaning christmas tree.
 * @param machine machine to check
 * @param firstRightBound step counter of the first right bound
 * @param secondRightBound step counter of the second right bound
 * @param thirdRightBound step counter of the third right bound
 * @param firstLeftBound pointer to the first left bound
 * @param secondLeftBound poiner to the second left bound
 * @param thirdLeftBound pointer to the third left bound
 * @param transition_limit max limit
 * @param N N component in the grammar
 * @param U U component in the grammar
 * @param X X component in the grammar
 * @param V V component in the grammar
 * @param s state s in the grammar
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
bool checkLeaningChristmasTreeGrammar(turing_machine<d_type> &machine,
							   unsigned int firstRightBound,
							   unsigned int secondRightBound,
							   unsigned int thirdRightBound,
							   std::list<d_type>::iterator firstLeftBound,
							   std::list<d_type>::iterator secondLeftBound,
							   std::list<d_type>::iterator thirdLeftBound,
							   unsigned int transition_limit,
							   tapeChunk<d_type> &N,
							   tapeChunk<d_type> &U,
							   tapeChunk<d_type> &X,
							   tapeChunk<d_type> &V,
							   std::vector<state<d_type> >::size_type s)
{
	machine.resetMachine();
	unsigned int step_counter = 0;
	tapeChunk<d_type> Xp, Y, Yp, Z, Vp, Vpp, Up, temp;
	std::vector<state<d_type> >::size_type q, r;

	for(step_counter; step_counter <= firstRightBound; step_counter++)
		machine.run_one();

	std::list<d_type>::iterator leftBound, rightBound;

	//XVs0* -> qX'V'0*
	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int j = 0; j < V.tapeConfig.size() + X.tapeConfig.size(); j++)
		leftBound--;
	temp = getNextChunk(machine,
						leftBound,
						rightBound,
						secondRightBound,
						step_counter,
						transition_limit,
						true,				//stop at left
						false,				//stop at right
						false,				//inclusive left
						true,				//inclusive right
						false,				//push left
						false);				//push right
	if(temp.isEmpty()) return false;
	q = machine.get_current_state();
	Xp.fillTape(temp, 0, X.tapeConfig.size()-1);
	Vp.fillTape(temp, X.tapeConfig.size(), temp.tapeConfig.size()-1);

	//std::cout << "here1" << std::endl;

	//NUqX' -> NNU'Y'r
	leftBound = firstLeftBound;
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int j = 0; j < Xp.tapeConfig.size()+1; j++)
		rightBound++;
	temp = getNextChunk(machine,
						leftBound,
						rightBound,
						secondRightBound,
						step_counter,
						transition_limit,
						false,				//stop at left
						true,				//stop at right
						true,				//inclusive left
						false,				//inclusive right
						true,				//push left
						false);				//push right
	if(temp.isEmpty()) return false;
	r = machine.get_current_state();
	Up.fillTape(temp, N.tapeConfig.size(), temp.tapeConfig.size()-X.tapeConfig.size()-1);
	Yp.fillTape(temp, temp.tapeConfig.size()-X.tapeConfig.size(), temp.tapeConfig.size()-1);
	if(temp != N+Up+Yp)
		return false;

	//std::cout << "here2" << std::endl;

	//Y'rV' -> ZVs''
	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int j = 0; j < Yp.tapeConfig.size(); j++)
		leftBound--;
	temp = getNextChunk(machine,
						leftBound,
						rightBound,
						secondRightBound,
						step_counter,
						transition_limit,
						false,				//stop at left
						false,				//stop at right
						true,				//inclusive left
						true,				//inclusive right
						false,				//push left
						true);				//push right
	if(temp.isEmpty()) return false;
	Z.fillTape(temp, 0, X.tapeConfig.size()-1);
	Vpp.fillTape(temp, X.tapeConfig.size(), temp.tapeConfig.size()-1);
	if(machine.get_current_state() != s)
		return false;

	/*std::cout << "Up : "; Up.print();
	std::cout << "Z  : "; Z.print();
	std::cout << "Vpp: "; Vpp.print();
	std::cout << "U :  "; U.print();
	std::cout << "X :  "; X.print();
	std::cout << "V :  "; V.print();
	std::cout << "N :  "; N.print();
	std::cout << "Xp : "; Xp.print();
	std::cout << "Yp : "; Yp.print();*/

	if(Up + Z + Vpp != U + X + X + V)
		return false;

	//XVs0* -> qX'V'0*
	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int j = 0; j < V.tapeConfig.size() + X.tapeConfig.size(); j++)
		leftBound--;
	temp = getNextChunk(machine,
						leftBound,
						rightBound,
						secondRightBound,
						step_counter,
						transition_limit,
						true,				//stop at left
						false,				//stop at right
						false,				//inclusive left
						true,				//inclusive right
						false,				//push left
						false);				//push right
	if(temp.isEmpty()) return false;
	if(temp != Xp + Vp || machine.get_current_state() != q)
		return false;

	//std::cout << "here3" << std::endl;

	//XqX' -> qX'Y
	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int j = 0; j < X.tapeConfig.size(); j++)
		leftBound--;
	for(int j = 0; j < Xp.tapeConfig.size(); j++)
		rightBound++;
	temp = getNextChunk(machine,
						leftBound,
						rightBound,
						secondRightBound,
						step_counter,
						transition_limit,
						true,				//stop at left
						false,				//stop at right
						false,				//inclusive left
						true,				//inclusive right
						false,				//push left
						false);				//push right
	if(temp.isEmpty()) return false;
	Y.fillTape(temp, Xp.tapeConfig.size(), temp.tapeConfig.size()-1);
	if(temp != Xp+Y || machine.get_current_state() != q)
		return false;

	//std::cout << "here4" << std::endl;

	//NUqX' -> NNU'Y'r
	leftBound = secondLeftBound;
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int j = 0; j < Xp.tapeConfig.size()+1; j++)
		rightBound++;
	temp = getNextChunk(machine,
						leftBound,
						rightBound,
						secondRightBound,
						step_counter,
						transition_limit,
						false,				//stop at left
						true,				//stop at right
						true,				//inclusive left
						false,				//inclusive right
						true,				//push left
						false);				//push right
	if(temp.isEmpty()) return false;
	if(temp != N+Up+Yp || machine.get_current_state() != r)
		return false;

	//std::cout << "here5" << std::endl;

	//Y'rY -> ZY'r
	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int j = 0; j < Yp.tapeConfig.size(); j++)
		leftBound--;
	for(int j = 0; j < Y.tapeConfig.size(); j++)
		rightBound++;
	temp = getNextChunk(machine,
						leftBound,
						rightBound,
						secondRightBound,
						step_counter,
						transition_limit,
						false,				//stop at left
						true,				//stop at right
						true,				//inclusive left
						false,				//inclusive right
						false,				//push left
						false);				//push right
	if(temp.isEmpty()) return false;
	if(temp != Z+Yp || machine.get_current_state() != r)
		return false;

	//std::cout << "here6" << std::endl;

	//Y'rV' -> ZVs''
	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int j = 0; j < Yp.tapeConfig.size(); j++)
		leftBound--;
	temp = getNextChunk(machine,
						leftBound,
						rightBound,
						thirdRightBound,
						step_counter,
						transition_limit,
						false,				//stop at left
						false,				//stop at right
						true,				//inclusive left
						true,				//inclusive right
						false,				//push left
						true);				//push right
	if(temp.isEmpty()) return false;
	if(temp != Z+Vpp || machine.get_current_state() != s)
		return false;
	if(Up + Z + Z + Vpp != U + X + X + X + V)
		return false;
	return true;
}

/**
 * specialized version of getNextRightBound for leaning christmas trees.
 * @post rightBound and leftBound are updated to the new values as well as step_counter
 * @param machine machine to check
 * @param rightBound location of previous right bound
 * @param step_counter current step of the machine
 * @param transition_limit point at which to stop searching
 * @return the step counter of the point at which the machine is in the next right most extremum
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 * @todo increase scope of this routine;  it currently does not work properly on select cases causing the leaning tree proof to fail when it should succeed
 */
unsigned int getNextLeaningRightBound(turing_machine<d_type> &machine,
							   std::list<d_type>::iterator &rightBound,
							   unsigned int &step_counter,
							   unsigned int transition_limit)
{
	bool rightBoundPushed = false, leftBoundPushed = false;
	unsigned int rightBoundStepCounter = 0;

	std::list<d_type>::iterator originalRightBound = rightBound;
	std::list<d_type>::iterator leftBound = rightBound;

	for(step_counter; step_counter < transition_limit; step_counter++)
	{
		if(machine.access_tape_read_write().currentLocation() == leftBound)
		{
			if(machine.nextMove() ==  infinite_tape<d_type>::left)
			{
				machine.access_tape_read_write().checkIteratorLeft(leftBound);
				leftBound--;
				leftBoundPushed = true;
				if(rightBoundPushed)
				{
					return rightBoundStepCounter;
				}
			}
			if(rightBoundPushed && leftBoundPushed)
			{
				return rightBoundStepCounter;
			}
		}
		if(machine.access_tape_read_write().currentLocation() == rightBound)
		{
			rightBoundPushed = true;
			rightBoundStepCounter = step_counter-1;
			if(machine.nextMove() ==  infinite_tape<d_type>::right)
			{
				machine.access_tape_read_write().checkIteratorRight(rightBound);
				rightBound++;
				rightBoundPushed = true;
				rightBoundStepCounter = step_counter;
			}
		}
		if(machine.run_one()==turing_machine<d_type>::halt)
			return 0;
	}
	return 0;
}

/**
 * leaning christmas tree non halt detection routine.
 * @param machine machine to check
 * @param transition_limit max limit before proof fails by default
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
bool checkLeaningChristmasTree(turing_machine<d_type> &machine, unsigned int transition_limit)
{
	machine.resetMachine();
	unsigned int step_counter = 0;

	std::list<d_type>::iterator leftBound;
	std::list<d_type>::iterator rightBound;

	//last bound push
	//0 = left
	//1 = right
	int lastPush = 0;

	//run for a bit to establish sweeping motion
	for(step_counter = 0; step_counter < transition_limit &&
						  step_counter < 200; step_counter++)
	{
		if(step_counter == 50)
		{
			leftBound = machine.access_tape_read_write().currentLocation();
			rightBound = machine.access_tape_read_write().currentLocation();
		}
		if(step_counter >= 50)
		{
			if(machine.access_tape_read_write().currentLocation() == rightBound)
				if(machine.nextMove() ==  infinite_tape<d_type>::right)
				{
					machine.access_tape_read_write().checkIteratorRight(rightBound);
					rightBound++;
				}
			if(machine.access_tape_read_write().currentLocation() == leftBound)
				if(machine.nextMove() ==  infinite_tape<d_type>::left)
				{
					machine.access_tape_read_write().checkIteratorLeft(leftBound);
					leftBound--;
				}
		}
		if(machine.run_one()==turing_machine<d_type>::halt)
			return false;

	}

	//get the step counters of the left and right bounds
	unsigned int firstRightBound = getNextLeaningRightBound(machine, rightBound, step_counter, transition_limit);
	if(firstRightBound == 0)
		return false;

	unsigned int secondRightBound = getNextLeaningRightBound(machine, rightBound, step_counter, transition_limit);
	if(secondRightBound == 0)
		return false;

	unsigned int thirdRightBound = getNextLeaningRightBound(machine, rightBound, step_counter, transition_limit);
	if(thirdRightBound == 0)
		return false;

	unsigned int fourthRightBound = getNextLeaningRightBound(machine, rightBound, step_counter, transition_limit);
	if(fourthRightBound == 0)
		return false;

	unsigned int fifthRightBound = getNextLeaningRightBound(machine, rightBound, step_counter, transition_limit);
	if(fifthRightBound == 0)
		return false;

	std::cout << firstRightBound << std::endl;
	std::cout << secondRightBound << std::endl;
	std::cout << thirdRightBound << std::endl;
	std::cout << fourthRightBound << std::endl;

	//reset the machine and use step counters to get tape configuration at each point
	machine.resetMachine();
	tapeChunk<d_type> chunkN, beforeSweep, afterSweep;

	std::vector<state<d_type> >::size_type stateS;
	std::list<d_type>::iterator firstLeftBound, secondLeftBound, thirdLeftBound, fourthLeftBound;
	std::list<d_type>::iterator tempHelper;

	for(unsigned int j = 0; j <= fifthRightBound; j++)
	{
		if(j < firstRightBound)
			machine.run_one();
		else if(j == firstRightBound)
		{
			firstLeftBound = machine.access_tape_read_write().currentLocation();
			machine.run_one();
		}
		else if(j > firstRightBound && j < secondRightBound)
		{
			if(machine.access_tape_read_write().currentLocation() == firstLeftBound)
				if(machine.nextMove() ==  infinite_tape<d_type>::left)
				{
					machine.access_tape_read_write().checkIteratorLeft(firstLeftBound);
					firstLeftBound--;
				}
			machine.run_one();
		}
		else if(j == secondRightBound)
		{
			machine.run_one();
			beforeSweep.fillTape(machine.access_tape_read_write(), firstLeftBound, machine.access_tape_read_write().currentLocation());
			secondLeftBound = machine.access_tape_read_write().currentLocation();
			stateS = machine.get_current_state();
		}
		else if(j > secondRightBound && j < thirdRightBound)
		{
			if(machine.access_tape_read_write().currentLocation() == secondLeftBound)
				if(machine.nextMove() ==  infinite_tape<d_type>::left)
				{
					machine.access_tape_read_write().checkIteratorLeft(secondLeftBound);
					secondLeftBound--;
				}
			machine.run_one();
		}
		else if(j == thirdRightBound)
		{
			machine.run_one();
			tempHelper = secondLeftBound;
			tempHelper--;
			chunkN.fillTape(machine.access_tape_read_write(), firstLeftBound, tempHelper);
			afterSweep.fillTape(machine.access_tape_read_write(), secondLeftBound, machine.access_tape_read_write().currentLocation());

			if(machine.get_current_state() != stateS)
				return false;
			thirdLeftBound = machine.access_tape_read_write().currentLocation();
		}
		else if(j > thirdRightBound && j < fourthRightBound)
		{
			if(machine.access_tape_read_write().currentLocation() == thirdLeftBound)
				if(machine.nextMove() ==  infinite_tape<d_type>::left)
				{
					machine.access_tape_read_write().checkIteratorLeft(thirdLeftBound);
					thirdLeftBound--;
				}
			machine.run_one();
		}
		else if(j == fourthRightBound)
		{
			machine.run_one();
			fourthLeftBound = machine.access_tape_read_write().currentLocation();
		}
		else
		{
			if(machine.access_tape_read_write().currentLocation() == fourthLeftBound)
				if(machine.nextMove() ==  infinite_tape<d_type>::left)
				{
					machine.access_tape_read_write().checkIteratorLeft(fourthLeftBound);
					fourthLeftBound--;
				}
			machine.run_one();
		}
	}


	tapeChunk<d_type> chunkX, chunkU, chunkV;

	if(!findMidSection(beforeSweep, afterSweep, chunkX, chunkU, chunkV))
		return false;

	chunkN.print();
	chunkX.print();
	chunkU.print();
	chunkV.print();

	if(!checkLeaningChristmasTreeGrammar(machine,
						thirdRightBound,
						fourthRightBound,
						fifthRightBound,
						secondLeftBound,
						thirdLeftBound,
						fourthLeftBound,
						transition_limit,
						chunkN,
						chunkU,
						chunkX,
						chunkV,
						stateS))
		return false;



	return true;
}

/**
 * leaning christmas tree non halt detection routine (checks mirror machines).
 * @param machine machine to check
 * @param transition_limit max limit before proof fails by default
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
bool leaningChristmasTree(turing_machine<d_type> &machine, unsigned int transition_limit)
{
	if(checkLeaningChristmasTree(machine, transition_limit))
		return true;
	else
	{
		turing_machine<d_type> reverse = machine.reverseMachine();
		if(checkLeaningChristmasTree(reverse, transition_limit))
			return true;
	}
	return false;
}

//end leaningChristmasTree routines



//begin counter routines

/**
 * get the next location of a blank cell in a counter.
 * @machine machine to check
 * @rightBound location of previous right bound
 * @leftBound location of previous left bound
 * @step_counter current step counter of machine
 * @transition_limit max limit to go to
 * @return the next point at which the state of the machine is 0*[E][Z]*[sB]
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
unsigned int getNextBlankLocation(turing_machine<d_type> &machine,
					std::list<d_type>::iterator &rightBound,
					std::list<d_type>::iterator &leftBound,
					unsigned int &step_counter,
					unsigned int transition_limit)
{
	bool rightBoundPushed = false;
	unsigned int rightBoundStepCounter = 0;

	for(step_counter; step_counter < transition_limit; step_counter++)
	{
		if(machine.access_tape_read_write().currentLocation() == rightBound)
		{
			if(machine.nextMove() ==  infinite_tape<d_type>::right)
			{
				machine.access_tape_read_write().checkIteratorRight(rightBound);
				rightBound++;
				if(!rightBoundPushed)
				{
					rightBoundStepCounter = step_counter;
					rightBoundPushed = true;
				}
			}
		}
		if(machine.access_tape_read_write().currentLocation() == leftBound)
		{
			if(machine.nextMove() ==  infinite_tape<d_type>::left)
				return 0;
			if(rightBoundPushed)
				return rightBoundStepCounter;
		}
		if(machine.run_one()==turing_machine<d_type>::halt)
			return 0;
	}
	return 0;
}

/**
 * check a transformation in the counter grammar.
 * @post the machine is run on the original tape using the original state as the starting state until either the left or right bound of the tape is breached
 * @param machine machine to check
 * @param originalTape original local tape to run the machine on
 * @param desiredTape hopeful result after machine is ru
 * @param originalReadHead original location of read head on local tape
 * @param desiredReadHead desired location of read head on local tape
 * @param originalState original starting state of machine for transformation
 * @param desiredState desired state of machine after transformation
 * @return true iff the desired conditions are the actual conditions on the machine
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
bool checkCounterConversion(turing_machine<d_type> &machine,
				tapeChunk<d_type> originalTape,
				tapeChunk<d_type> desiredTape,
				int originalReadHead,
				int desiredReadHead,
				std::vector<state<d_type> >::size_type originalState,
				std::vector<state<d_type> >::size_type desiredState,
				std::vector<state<d_type> >::size_type &alternateDesiredState,
				bool setAlternate)
{
	std::list<d_type>::iterator leftBound;
	std::list<d_type>::iterator rightBound;
	std::list<d_type>::iterator desiredLocation;
	machine.rigMachine(originalTape.tapeConfig, originalReadHead, desiredReadHead, originalState, leftBound, rightBound, desiredLocation);

	for(int j = 0; j < 100; j++)
	{
		if(machine.access_tape_read_write().currentLocation() == leftBound)
			if(machine.nextMove() ==  infinite_tape<d_type>::left)
			{
				machine.run_one();
				break;
			}
		if(machine.access_tape_read_write().currentLocation() == rightBound)
			if(machine.nextMove() ==  infinite_tape<d_type>::right)
			{
				machine.run_one();
				break;
			}
		machine.run_one();
	}

	tapeChunk<d_type> result;
	result.fillTape(machine.access_tape_read_write(), leftBound, rightBound);

	/*result.print();
	desiredTape.print();
	std::cout << machine.get_current_state() << " " << desiredState << std::endl;
	std::cout << std::endl;*/

	if(setAlternate)
		alternateDesiredState = machine.get_current_state();

	if(machine.access_tape_read_write().currentLocation() == desiredLocation &&
	   (machine.get_current_state() == desiredState ||
	    machine.get_current_state() == alternateDesiredState) &&
	   result == desiredTape)
		return true;
	return false;
}

/**
 * checks all of the necessary transformations in the counter grammar.
 * @param machine machine to check
 * @param splitLeftSide left side resulting run when counter is run from the blank location to halfway to the end cell.
 * @param splitLeftSide right side resulting run when counter is run from the blank location to halfway to the end cell.
 * @param zeroCell zero component
 * @param oneCell one component
 * @param onePrimeCell one prime component
 * @param endCell end component
 * @param cellSize size of the above components
 * @param stateC carry signal
 * @param stateR return signal
 * @return true if the split sides have the correct components and if all of the counter grammar transformations hold
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
bool checkFinalCounterGrammar(turing_machine<d_type> &machine,
				tapeChunk<d_type> splitLeftSide,
				tapeChunk<d_type> splitRightSide,
				tapeChunk<d_type> zeroCell,
				tapeChunk<d_type> oneCell,
				tapeChunk<d_type> onePrimeCell,
				tapeChunk<d_type> endCell,
				int cellSize,
				std::vector<state<d_type> >::size_type stateC,
				std::vector<state<d_type> >::size_type stateR)
{
	int index = 0;
	while(index < splitLeftSide.length())
	{
		if(index == 0)
		{
			if(!splitLeftSide.compareNextChunk(endCell, index))
				return false;
		}
		else
		{
			if(!splitLeftSide.compareNextChunk(onePrimeCell, index))
				return false;
		}
	}
	index = 0;
	while(index < splitRightSide.length())
	{
		if(index + cellSize >= splitRightSide.length())
		{
			if(!splitRightSide.compareNextChunk(oneCell, index))
				return false;
		}
		else
		{
			if(!splitRightSide.compareNextChunk(zeroCell, index))
				return false;
		}
	}

	//std::cout << "here" << std::endl;

	std::vector<state<d_type> >::size_type alternateR;

	tapeChunk<d_type> onePrimeZero, zeroOne;
	onePrimeZero = onePrimeCell + zeroCell;
	zeroOne = zeroCell + oneCell;

	tapeChunk<d_type> onePrimeOne, onePrimeOnePrime;
	onePrimeOne = onePrimeCell + oneCell;
	onePrimeOnePrime = onePrimeCell + onePrimeCell;

	tapeChunk<d_type> zeroZero;
	zeroZero = zeroCell + zeroCell;

	tapeChunk<d_type> endOne, endZero, endOnePrime;
	endOne = endCell + oneCell;
	endZero = endCell + zeroCell;
	endOnePrime = endCell + onePrimeCell;

	if(//checkCounterConversion(machine, onePrimeCell, zeroCell, cellSize - 1, -1, stateR, stateR, alternateR, true) &&
	   //checkCounterConversion(machine, onePrimeCell, zeroCell, cellSize - 1, -1, alternateR, stateR, alternateR, false) &&
	   checkCounterConversion(machine, onePrimeOne, zeroOne, cellSize - 1, -1, stateR, stateR, alternateR, true) &&
	   checkCounterConversion(machine, onePrimeZero, zeroZero, cellSize - 1, -1, alternateR, stateR, alternateR, false) &&
	   //checkCounterConversion(machine, endCell, endCell, endCell.length()-1, endCell.length(), stateR, stateC, stateC, false) &&
	   checkCounterConversion(machine, endOne, endOnePrime, endCell.length()-1, endOnePrime.length(), stateR, stateC, stateC, false) &&
	   checkCounterConversion(machine, endZero, endOnePrime, endCell.length()-1, endOnePrime.length(), stateR, stateC, stateC, false) &&
	   checkCounterConversion(machine, onePrimeOne, onePrimeOnePrime, onePrimeCell.length(), onePrimeOne.length(), stateC, stateC, stateC, false) &&
	   checkCounterConversion(machine, onePrimeZero, zeroOne, onePrimeCell.length(), -1, stateC, stateR, alternateR, false))
	   //checkCounterConversion(machine, onePrimeOnePrime, zeroZero, onePrimeOnePrime.length() - 1, -1, stateR, stateR) &&
	   //checkCounterConversion(machine, oneCell, onePrimeCell, 0, cellSize, stateC, stateC) &&
	   //checkCounterConversion(machine, zeroCell, oneCell, 0, -1, stateC, stateR))
		return true;
	return false;
}

/**
 * check the run of a counter from a blank location to verify transformations.
 * @post machine is run from the firstBlankLocation until it is halfway down the tape;  the components at each side of the read head are then checked and the transformations in the grammar are confirmed; if all holds, return true else false
 * @param machine machine to check
 * @param transition_limit max limit
 * @param firstBlankLocationCounter step counter of the first blank location
 * @param firstBlankLocation entire tape at the point of the first blank location counter
 * @param zeroCell zero cell
 * @param oneCell one cell
 * @param onePrimeCell one prime cell
 * @param endCell end cell
 * @param cellSize size of the above components
 * @param stateC carry signal
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
bool checkCounterRun(turing_machine<d_type> &machine,
			unsigned int transition_limit,
			unsigned int firstBlankLocationCounter,
			tapeChunk<d_type> firstBlankLocation,
			tapeChunk<d_type> zeroCell,
			tapeChunk<d_type> oneCell,
			tapeChunk<d_type> onePrimeCell,
			tapeChunk<d_type> endCell,
			unsigned int cellSize,
			std::vector<state<d_type> >::size_type stateC)
{
	int currentLocation = firstBlankLocation.length() - 1, onePrimeCount = 0;
	tapeChunk<d_type> temp(onePrimeCell);

	/*std::cout << "test : ";
	firstBlankLocation.print();*/

	while(currentLocation - cellSize >= endCell.length() && temp == onePrimeCell)
	{
		temp.emptyTape();
		temp.fillTape(firstBlankLocation, currentLocation - cellSize, currentLocation - 1);
		onePrimeCount++;
		currentLocation -= cellSize;
	}

	//std::cout << "oneprimes : " << onePrimeCount << std::endl;

	unsigned int snipLocation = cellSize*(onePrimeCount/2) + 1;

	machine.resetMachine();
	unsigned int step_counter = 0;
	std::list<d_type>::iterator leftBound = machine.access_tape_read_write().currentLocation();
	std::list<d_type>::iterator rightBound = machine.access_tape_read_write().currentLocation();
	std::list<d_type>::iterator snipSpot;
	tapeChunk<d_type> leftSide, rightSide;
	std::vector<state<d_type> >::size_type stateR;


	for(step_counter; step_counter <= firstBlankLocationCounter; step_counter++)
	{
		if(step_counter == 50)
		{
			leftBound = machine.access_tape_read_write().currentLocation();
			rightBound = machine.access_tape_read_write().currentLocation();
		}
		if(step_counter >= 50)
		{
			if(machine.access_tape_read_write().currentLocation() == leftBound)
				if(machine.nextMove() ==  infinite_tape<d_type>::left)
				{
					machine.access_tape_read_write().checkIteratorLeft(leftBound);
					leftBound--;
				}
			if(machine.access_tape_read_write().currentLocation() == rightBound)
				if(machine.nextMove() ==  infinite_tape<d_type>::right)
				{
					machine.access_tape_read_write().checkIteratorRight(rightBound);
					rightBound++;
				}
		}
		machine.run_one();
	}
	snipSpot = rightBound;
	for(unsigned int j = 0; j < snipLocation; j++)
		snipSpot--;

	while(step_counter < transition_limit && machine.access_tape_read_write().currentLocation() != snipSpot)
	{
		if(machine.access_tape_read_write().currentLocation() == leftBound)
			if(machine.nextMove() ==  infinite_tape<d_type>::left)
			{
				return false;
			}
		if(machine.access_tape_read_write().currentLocation() == rightBound)
			if(machine.nextMove() ==  infinite_tape<d_type>::right)
			{
				machine.access_tape_read_write().checkIteratorRight(rightBound);
				rightBound++;
			}
		machine.run_one();
		step_counter++;
	}

	if(step_counter == transition_limit)
		return false;

	stateR = machine.get_current_state();
	leftSide.fillTape(machine.access_tape_read_write(), leftBound, snipSpot);
	rightSide.fillTape(machine.access_tape_read_write(), ++snipSpot, rightBound);

	/*std::cout << std::endl;
	leftSide.print();
	rightSide.print();*/

	if(!checkFinalCounterGrammar(machine,
					leftSide,
					rightSide,
					zeroCell,
					oneCell,
					onePrimeCell,
					endCell,
					cellSize,
					stateC,
					stateR))
		return false;

	return true;
}

/**
 * extract the components in a counter.
 * @param machine machine to use
 * @param firstBlankLocationCounter step counter of first blank location
 * @param firstBlankLocation entire tape at the firstBlankLocationCounter
 * @param secondBlankLocation entire tape at the secondBlankLocation step counter
 * @param middleLocation entire tape at the point where the read head is at the left most point between the first and second blank location
 * @param stateC carry signal
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
bool extractCounterElements(turing_machine<d_type> &machine,
				unsigned int transition_limit,
				unsigned int firstBlankLocationCounter,
				tapeChunk<d_type> firstBlankLocation,
				tapeChunk<d_type> secondBlankLocation,
				tapeChunk<d_type> middleLocation,
				std::vector<state<d_type> >::size_type stateC)
{
	int cellSize = secondBlankLocation.length() - firstBlankLocation.length();
	if(cellSize < 0 || middleLocation.length() < 2*cellSize + 1 || firstBlankLocation.length() < cellSize + 1)
		return false;

	tapeChunk<d_type> zeroCell, oneCell, onePrimeCell, endCell;
	tapeChunk<d_type> temp;

	oneCell.fillTape(middleLocation, middleLocation.length() - cellSize, middleLocation.length() - 1);
	zeroCell.fillTape(middleLocation, middleLocation.length() - 2*cellSize, middleLocation.length() - cellSize - 1);
	onePrimeCell.fillTape(firstBlankLocation, firstBlankLocation.length() - cellSize - 1, firstBlankLocation.length() - 2);

	int currentLocation = firstBlankLocation.length() - cellSize - 1;
	do
	{
		temp.emptyTape();
		temp.fillTape(firstBlankLocation, currentLocation - cellSize, currentLocation - 1);
		currentLocation -= cellSize;
	}while(currentLocation - cellSize > 0 && temp == onePrimeCell);

	endCell.fillTape(firstBlankLocation, 0, currentLocation + cellSize + 1);

	/*zeroCell.print();
	oneCell.print();
	onePrimeCell.print();
	endCell.print();*/

	if(!checkCounterRun(machine,
				transition_limit,
				firstBlankLocationCounter,
				firstBlankLocation,
				zeroCell,
				oneCell,
				onePrimeCell,
				endCell,
				cellSize,
				stateC))
		return false;

	return true;
}

/**
 * test if the machine is a counter
 * @return true iff the machine exhibits counter behavior
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
bool testCounter(turing_machine<d_type> &machine, unsigned int transition_limit)
{
	machine.resetMachine();
	unsigned int step_counter = 0;

	std::list<d_type>::iterator leftBound = machine.access_tape_read_write().currentLocation();
	std::list<d_type>::iterator rightBound = machine.access_tape_read_write().currentLocation();

	//run for a bit to account for startup
	for(step_counter = 0; step_counter < transition_limit &&
						  step_counter < 700; step_counter++)
	{
		if(step_counter == 50)
		{
			leftBound = machine.access_tape_read_write().currentLocation();
			rightBound = machine.access_tape_read_write().currentLocation();
		}
		if(step_counter >= 50)
		{
			if(machine.access_tape_read_write().currentLocation() == rightBound)
				if(machine.nextMove() ==  infinite_tape<d_type>::right)
				{
					machine.access_tape_read_write().checkIteratorRight(rightBound);
					rightBound++;
				}
			if(machine.access_tape_read_write().currentLocation() == leftBound)
				if(machine.nextMove() ==  infinite_tape<d_type>::left)
				{
					machine.access_tape_read_write().checkIteratorLeft(leftBound);
					leftBound--;
				}
		}
		if(machine.run_one()==turing_machine<d_type>::halt)
			return false;
	}

	unsigned int firstBlankLocation = getNextBlankLocation(machine, rightBound, leftBound, step_counter, transition_limit);
	if(firstBlankLocation == 0)
		return false;
	unsigned int firstLeftBound = step_counter - 1;

	unsigned int secondBlankLocation = getNextBlankLocation(machine, rightBound, leftBound, step_counter, transition_limit);
	if(secondBlankLocation == 0)
		return false;

	machine.resetMachine();
	tapeChunk<d_type> firstTape, middleTape, secondTape;
	std::vector<state<d_type> >::size_type stateC, stateM;


	for(unsigned int j = 0; j <= secondBlankLocation; j++)
	{
		if(machine.access_tape_read_write().currentLocation() == leftBound)
			if(machine.nextMove() ==  infinite_tape<d_type>::left)
			{
				machine.access_tape_read_write().checkIteratorLeft(leftBound);
				leftBound--;
			}
		if(machine.access_tape_read_write().currentLocation() == rightBound)
			if(machine.nextMove() ==  infinite_tape<d_type>::right)
			{
				machine.access_tape_read_write().checkIteratorRight(rightBound);
				rightBound++;
			}
		machine.run_one();

		if(j == firstBlankLocation)
		{
			rightBound = machine.access_tape_read_write().currentLocation();
			firstTape.fillTape(machine.access_tape_read_write(), leftBound, rightBound);
			stateC = machine.get_current_state();
		}
		else if(j == firstLeftBound)
		{
			middleTape.fillTape(machine.access_tape_read_write(), leftBound, rightBound);
			stateM = machine.get_current_state();
		}
		else if(j == secondBlankLocation)
		{
			rightBound = machine.access_tape_read_write().currentLocation();
			secondTape.fillTape(machine.access_tape_read_write(), leftBound, rightBound);
			if(stateC != machine.get_current_state())
				return false;
		}
	}

	//std::cout << firstBlankLocation << std::endl;
	//std::cout << secondBlankLocation << std::endl;

	/*firstTape.print();
	middleTape.print();
	secondTape.print();
	std::cout << std::endl;*/

	if(!extractCounterElements(machine, transition_limit, firstBlankLocation, firstTape, secondTape, middleTape, stateC))
		return false;

	return true;
}

/**
 * test if the machine is a counter
 * @return true iff the machine exhibits counter behavior
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
bool counter(turing_machine<d_type> &machine, unsigned int transition_limit)
{
	if(testCounter(machine, transition_limit))
		return true;
	else
	{
		turing_machine<d_type> reverse = machine.reverseMachine();
		if(testCounter(reverse, transition_limit))
			return true;
	}
	return false;
}

//end counter routines

//general multisweep tree routines

/**
 * run a machine for a while to establish a sweeping motion
 * @param machine machine to run
 * @param leftBound update the left bound pointer as you go
 * @param rightBound update the right bound pointer as you go
 * @param step_counter update the step counter as you go
 * @param step_limit steps to run for
 * @param transition_limit max steps
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
bool establishSweep(turing_machine<d_type> &machine,
			std::list<d_type>::iterator &leftBound,
			std::list<d_type>::iterator &rightBound,
			unsigned int &step_counter,
			unsigned int step_limit,
			unsigned int transition_limit)
{
	for(step_counter; step_counter < transition_limit &&
						  step_counter < step_limit; step_counter++)
	{
		if(step_counter == 50)
		{
			leftBound = machine.access_tape_read_write().currentLocation();
			rightBound = machine.access_tape_read_write().currentLocation();
		}
		if(step_counter >= 50)
		{
			if(machine.access_tape_read_write().currentLocation() == rightBound)
				if(machine.nextMove() ==  infinite_tape<d_type>::right)
				{
					machine.access_tape_read_write().checkIteratorRight(rightBound);
					rightBound++;
				}
			if(machine.access_tape_read_write().currentLocation() == leftBound)
				if(machine.nextMove() ==  infinite_tape<d_type>::left)
				{
					machine.access_tape_read_write().checkIteratorLeft(leftBound);
					leftBound--;
				}
		}
		if(machine.run_one()==turing_machine<d_type>::halt)
			return false;

	}
	return true;
}

/**
 * check the first pass through the grammar in a multi-sweep christmas tree.
 * @param machine machine to check
 * @param currentSweep current sweep of the machine
 * @param totalSweeps total sweeps to check
 * @param rightBoundCounters vector of right bounds representing the step counter for the right bound at each sweep
 * @param leftBoundPointers vector of left bounds representing the pointers to the cell location for the left bounds at each sweep
 * @param transition_limit max steps
 * @param step_counter current step of the machine
 * @param U U component of the grammar
 * @param X X component of the grammar
 * @param V V component of the grammar
 * @param s state s in the grammar
 * @param statistics represents stats that maintains information about previous sweeps
 * @return true iff the grammar is satisified for one complete pass
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
bool checkMultiSweepFirstPass(turing_machine<d_type> &machine,
				unsigned int currentSweep,
				unsigned int totalSweeps,
				std::vector<unsigned int> rightBoundCounters,
				std::vector< std::list<d_type>::iterator > leftBoundPointers,
				unsigned int transition_limit,
				unsigned int &step_counter,
				tapeChunk<d_type> &U,
				tapeChunk<d_type> &X,
				tapeChunk<d_type> &V,
				std::vector<state<d_type> >::size_type s,
				std::vector<sweepStats<d_type> > &statistics)
{
	tapeChunk<d_type> Xp, Y, Yp, Z, Vp, Vpp, Up, temp;
	std::vector<state<d_type> >::size_type q, r, t;
	sweepStats<d_type> myStats;

	statistics.push_back(myStats);

	if(currentSweep == 1)
		for(step_counter; step_counter <= rightBoundCounters[totalSweeps]; step_counter++)
			machine.run_one();

	std::list<d_type>::iterator leftBound, rightBound;

	//XVs0* -> qX'V'0*
	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int j = 0; j < V.tapeConfig.size() + X.tapeConfig.size(); j++)
		leftBound--;
	temp = getNextChunk(machine,
						leftBound,
						rightBound,
						0,
						step_counter,
						transition_limit,
						true,				//stop at left
						false,				//stop at right
						false,				//inclusive left
						true,				//inclusive right
						false,				//push left
						false);				//push right
	if(temp.isEmpty()) return false;
	q = machine.get_current_state();
	Xp.fillTape(temp, 0, X.tapeConfig.size()-1);
	Vp.fillTape(temp, X.tapeConfig.size(), temp.tapeConfig.size()-1);

	//0*UqX' -> 0*U'Y'r
	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int j = 0; j < Xp.tapeConfig.size()+1; j++)
		rightBound++;
	temp = getNextChunkUnevenCheck(machine,
						leftBoundPointers[currentSweep-1],
						leftBound,
						rightBound,
						0,
						step_counter,
						transition_limit,
						false,				//stop at left
						true,				//stop at right
						true,				//inclusive left
						false,				//inclusive right
						true,				//push left
						false);				//push right
	if(temp.isEmpty()) return false;
	r = machine.get_current_state();
	Up.fillTape(temp, 0, temp.tapeConfig.size()-X.tapeConfig.size()-1);
	Yp.fillTape(temp, temp.tapeConfig.size()-X.tapeConfig.size(), temp.tapeConfig.size()-1);

	//Y'rV' -> ZVs''
	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int j = 0; j < Yp.tapeConfig.size(); j++)
		leftBound--;
	temp = getNextChunk(machine,
						leftBound,
						rightBound,
						rightBoundCounters[currentSweep+totalSweeps],
						step_counter,
						transition_limit,
						false,				//stop at left
						false,				//stop at right
						true,				//inclusive left
						true,				//inclusive right
						false,				//push left
						true);				//push right
	if(temp.isEmpty()) return false;
	t = machine.get_current_state();
	Z.fillTape(temp, 0, X.tapeConfig.size()-1);
	Vpp.fillTape(temp, X.tapeConfig.size(), temp.tapeConfig.size()-1);

	statistics[statistics.size()-1].fill(U, V, X, Xp, Y, Yp, Z, Vp, Vpp, Up, q, r, t);

	/*std::cout << "U  : "; U.print();
	std::cout << "V  : "; V.print();
	std::cout << "X  : "; X.print();
	std::cout << "Xp : "; Xp.print();
	std::cout << "Y  : "; Y.print();
	std::cout << "Yp : "; Yp.print();
	std::cout << "Z  : "; Z.print();
	std::cout << "Vp : "; Vp.print();
	std::cout << "Vpp: "; Vpp.print();
	std::cout << "Up : "; Up.print();
	std::cout << "q  : " << q << std::endl;
	std::cout << "r  : " << r << std::endl;
	std::cout << "s  : " << s << std::endl;
	std::cout << "t  : " << t << std::endl;

	tapeChunk<d_type> temp1 = statistics[0].U + statistics[0].X + statistics[0].X + statistics[0].V;
	tapeChunk<d_type> temp2 = Up + Z + Vpp;
	std::cout << "U+X+X+V : "; temp1.print();
	std::cout << "Up+Z+Vpp: "; temp2.print();*/

	if(currentSweep == totalSweeps)
	{
		if(t != s || Up + Z + Vpp != statistics[0].U + statistics[0].X + statistics[0].X + statistics[0].V)
			return false;
	}
	else
	{
		return checkMultiSweepFirstPass(machine, currentSweep+1, totalSweeps, rightBoundCounters, leftBoundPointers,
						transition_limit, step_counter, Up, Z, Vpp, s, statistics);
	}
	return true;
}

/**
 * check the first pass through the grammar in a multi-sweep christmas tree.
 * @param machine machine to check
 * @param currentSweep current sweep of the machine
 * @param totalSweeps total sweeps to check
 * @param rightBoundCounters vector of right bounds representing the step counter for the right bound at each sweep
 * @param leftBoundPointers vector of left bounds representing the pointers to the cell location for the left bounds at each sweep
 * @param transition_limit max steps
 * @param step_counter current step of the machine
 * @param s state s in the grammar
 * @param statistics represents stats that maintains information about previous sweeps
 * @return true iff the grammar is satisified for the second complete pass
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
bool checkMultiSweepSecondPass(turing_machine<d_type> &machine,
					unsigned int currentSweep,
					unsigned int totalSweeps,
					std::vector<unsigned int> rightBoundCounters,
					std::vector< std::list<d_type>::iterator > leftBoundPointers,
					unsigned int transition_limit,
					unsigned int &step_counter,
					std::vector<state<d_type> >::size_type s,
					std::vector<sweepStats<d_type> > &statistics)
{
	tapeChunk<d_type> U, V, X, Xp, Y, Yp, Z, Vp, Vpp, Up, temp;
	std::vector<state<d_type> >::size_type q, r, t;

	statistics[currentSweep-1].populate(U, V, X, Xp, Y, Yp, Z, Vp, Vpp, Up, q, r, t);

	std::list<d_type>::iterator leftBound, rightBound;

	//XVs0* -> qX'V'0*
	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int j = 0; j < V.tapeConfig.size() + X.tapeConfig.size(); j++)
		leftBound--;
	temp = getNextChunk(machine,
						leftBound,
						rightBound,
						0,
						step_counter,
						transition_limit,
						true,				//stop at left
						false,				//stop at right
						false,				//inclusive left
						true,				//inclusive right
						false,				//push left
						false);				//push right
	if(temp.isEmpty()) return false;
	if(temp != Xp + Vp || machine.get_current_state() != q)
		return false;

	//XqX' -> qX'Y
	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int j = 0; j < X.tapeConfig.size(); j++)
		leftBound--;
	for(int j = 0; j < Xp.tapeConfig.size(); j++)
		rightBound++;
	temp = getNextChunk(machine,
						leftBound,
						rightBound,
						0,
						step_counter,
						transition_limit,
						true,				//stop at left
						false,				//stop at right
						false,				//inclusive left
						true,				//inclusive right
						false,				//push left
						false);				//push right
	if(temp.isEmpty()) return false;
	Y.fillTape(temp, Xp.tapeConfig.size(), temp.tapeConfig.size()-1);
	statistics[currentSweep-1].Y.fillTape(temp, Xp.tapeConfig.size(), temp.tapeConfig.size()-1);
	if(temp != Xp+Y || machine.get_current_state() != q)
		return false;

	//0*UqX' -> 0*U'Y'r
	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int j = 0; j < Xp.tapeConfig.size()+1; j++)
		rightBound++;
	temp = getNextChunkUnevenCheck(machine,
						leftBoundPointers[currentSweep+totalSweeps-1],
						leftBound,
						rightBound,
						0,
						step_counter,
						transition_limit,
						false,				//stop at left
						true,				//stop at right
						true,				//inclusive left
						false,				//inclusive right
						true,				//push left
						false);				//push right
	if(temp.isEmpty()) return false;
	if(temp != Up+Yp || machine.get_current_state() != r)
		return false;

	//Y'rY -> ZY'r
	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int j = 0; j < Yp.tapeConfig.size(); j++)
		leftBound--;
	for(int j = 0; j < Y.tapeConfig.size(); j++)
		rightBound++;
	temp = getNextChunk(machine,
						leftBound,
						rightBound,
						0,
						step_counter,
						transition_limit,
						false,				//stop at left
						true,				//stop at right
						true,				//inclusive left
						false,				//inclusive right
						false,				//push left
						false);				//push right
	if(temp.isEmpty()) return false;
	if(temp != Z+Yp || machine.get_current_state() != r)
		return false;

	//Y'rV' -> ZVs''
	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int j = 0; j < Yp.tapeConfig.size(); j++)
		leftBound--;
	temp = getNextChunk(machine,
						leftBound,
						rightBound,
						rightBoundCounters[currentSweep+2*totalSweeps],
						step_counter,
						transition_limit,
						false,				//stop at left
						false,				//stop at right
						true,				//inclusive left
						true,				//inclusive right
						false,				//push left
						true);				//push right
	if(temp.isEmpty()) return false;
	if(temp != Z+Vpp || machine.get_current_state() != t)
		return false;

	/*std::cout << "Up : "; Up.print();
	std::cout << "Z  : "; Z.print();
	std::cout << "Vpp: "; Vpp.print();
	std::cout << "U :  "; U.print();
	std::cout << "X :  "; X.print();
	std::cout << "V :  "; V.print();*/

	if(currentSweep == totalSweeps)
	{
		if(t != s || Up + Z + Z + Vpp != statistics[0].U + statistics[0].X + statistics[0].X + statistics[0].X + statistics[0].V)
			return false;
	}
	else
	{
		return checkMultiSweepSecondPass(machine, currentSweep+1, totalSweeps, rightBoundCounters, leftBoundPointers,
						transition_limit, step_counter, s, statistics);
	}
	return true;
}

/**
 * check the grammar for christmas trees.
 * @param machine machine to check
 * @param totalSweeps number of sweeps in this christmas tree grammar check
 * @param rightBoundCounters vector of right bounds representing the step counter for the right bound at each sweep
 * @param leftBoundPointers vector of left bounds representing the pointers to the cell location for the left bounds at each sweep
 * @param transition_limit max steps
 * @param U U component of the grammar
 * @param X X component of the grammar
 * @param V V component of the grammar
 * @param s state s in the grammar
 * @return true iff the grammar is satisfied for all transformations
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
bool checkMultiSweepChristmasTreeGrammar(turing_machine<d_type> &machine,
					unsigned int totalSweeps,
					std::vector<unsigned int> rightBoundCounters,
					std::vector< std::list<d_type>::iterator > leftBoundPointers,
					unsigned int transition_limit,
					tapeChunk<d_type> &U,
					tapeChunk<d_type> &X,
					tapeChunk<d_type> &V,
					std::vector<state<d_type> >::size_type s)
{
	machine.resetMachine();

	unsigned int step_counter = 0;
	std::vector<sweepStats<d_type> > statistics(0);

	if(checkMultiSweepFirstPass(machine, 1, totalSweeps, rightBoundCounters, leftBoundPointers, transition_limit, step_counter, U, X, V, s, statistics) &&
		checkMultiSweepSecondPass(machine, 1, totalSweeps, rightBoundCounters, leftBoundPointers, transition_limit, step_counter, s, statistics))
		return true;
	return false;
}

/**
 * check for christmas tree non-halter.
 * @param machine machine to check
 * @param sweeps number of sweeps to check in the multi-sweep grammar
 * @param transition_limit max steps before fail
 * @return true iff machine is sweeps-sweep christmas tree
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
bool checkMultiSweepChristmasTree(turing_machine<d_type> &machine, unsigned int sweeps, unsigned int transition_limit)
{
	machine.resetMachine();
	unsigned int step_counter = 0;

	std::list<d_type>::iterator leftBound;
	std::list<d_type>::iterator rightBound;

	std::vector<unsigned int> rightBoundCounters;
	std::vector< std::list<d_type>::iterator > leftBoundPointers;

	//run for a bit to establish sweeping motion
	if(!establishSweep(machine, leftBound, rightBound, step_counter, 150, transition_limit))
		return false;

	//get step counters for each right bound
	unsigned int temp;
	for(int j = 0; j < 3*sweeps + 1; j++)
	{
		temp = getNextRightBound(machine, rightBound, leftBound, step_counter, transition_limit);
		if(temp == 0)
			return false;
		rightBoundCounters.push_back(temp);
	}

	//reset the machine and use step counters to get tape configuration at each point
	machine.resetMachine();
	tapeChunk<d_type> beforeSweep, afterSweep;

	std::vector<state<d_type> >::size_type stateS;

	for(unsigned int j = 0; j <= rightBoundCounters[rightBoundCounters.size()-1]; j++)
	{
		if(j < 50)
			machine.run_one();
		if(j == 50)
			leftBound = machine.access_tape_read_write().currentLocation();
		if(j >= 50)
		{
			if(machine.access_tape_read_write().currentLocation() == leftBound)
				if(machine.nextMove() ==  infinite_tape<d_type>::left)
				{
					machine.access_tape_read_write().checkIteratorLeft(leftBound);
					leftBound--;
				}
			machine.run_one();

			if(j == rightBoundCounters[0])
			{
				rightBound = machine.access_tape_read_write().currentLocation();
				beforeSweep.fillTape(machine.access_tape_read_write(), leftBound, rightBound);
				stateS = machine.get_current_state();
			}

			else if(j == rightBoundCounters[sweeps])
			{
				rightBound = machine.access_tape_read_write().currentLocation();
				afterSweep.fillTape(machine.access_tape_read_write(), leftBound, rightBound);
				if(machine.get_current_state() != stateS)
					return false;
			}

			else
			{
				for(int i = sweeps+1; i < rightBoundCounters.size(); i++)
					if(j == rightBoundCounters[i])
						leftBoundPointers.push_back(leftBound);
			}
		}
	}


	tapeChunk<d_type> chunkX, chunkU, chunkV;

	if(!findMidSection(beforeSweep, afterSweep, chunkX, chunkU, chunkV))
		return false;

	/*for(int i = 0; i < rightBoundCounters.size(); i++)
		std::cout << rightBoundCounters[i] << std::endl;
	chunkX.print();
	chunkU.print();
	chunkV.print();*/

	if(!checkMultiSweepChristmasTreeGrammar(machine, sweeps, rightBoundCounters, leftBoundPointers, transition_limit, chunkU, chunkX, chunkV, stateS))
		return false;



	return true;
}

/**
 * check for christmas tree non-halter (mirror machines as well).
 * @param machine machine to check
 * @param sweeps number of sweeps to check in the multi-sweep grammar
 * @param transition_limit max steps before fail
 * @return true iff machine is sweeps-sweep christmas tree
 * @author Owen Kellett
 * @ingroup NonHaltDetectionRoutines
 */
bool multiSweepChristmasTree(turing_machine<d_type> &machine, unsigned int sweeps, unsigned int transition_limit)
{
	if(checkMultiSweepChristmasTree(machine, sweeps, transition_limit))
		return true;
	else
	{
		turing_machine<d_type> reverse = machine.reverseMachine();
		if(checkMultiSweepChristmasTree(reverse, sweeps, transition_limit))
			return true;
	}
	return false;
}

//end general multisweep tree routines

//begin nested christmas tree routines

unsigned int getNextNestedRightBound(turing_machine<d_type> &machine,
							   std::list<d_type>::iterator &rightBound,
							   std::list<d_type>::iterator &leftBound,
							   unsigned int &step_counter,
							   unsigned int transition_limit)
{
	bool rightBoundPushed = false;
	unsigned int rightBoundStepCounter = 0;

	for(step_counter; step_counter < transition_limit; step_counter++)
	{
		if(machine.access_tape_read_write().currentLocation() == rightBound)
		{
			if(!rightBoundPushed)
			{
				rightBoundPushed = true;
				rightBoundStepCounter = step_counter-1;
			}
			if(machine.nextMove() ==  infinite_tape<d_type>::right)
			{
				machine.access_tape_read_write().checkIteratorRight(rightBound);
				rightBound++;
				rightBoundPushed = true;
				rightBoundStepCounter = step_counter;
			}
		}
		if(machine.access_tape_read_write().currentLocation() == leftBound)
		{
			if(machine.nextMove() ==  infinite_tape<d_type>::left)
			{
				machine.access_tape_read_write().checkIteratorLeft(leftBound);
				leftBound--;
				if(rightBoundPushed)
				{
					return rightBoundStepCounter;
				}
			}
			/*if(rightBoundPushed)
			{
				return rightBoundStepCounter;
			}*/
		}
		if(machine.run_one()==turing_machine<d_type>::halt)
			return 0;
	}
	return 0;
}

int extractws(tapeChunk<d_type> &NV,
		tapeChunk<d_type> &Y,
		tapeChunk<d_type> &Vp,
		tapeChunk<d_type> &W,
		tapeChunk<d_type> &Vend,
		int offset)
{
	tapeChunk<d_type> Yend, totalEnd;
	Yend.fillTape(Y, offset, Y.tapeConfig.size()-1);
	totalEnd = Yend + Vp;
	W.fillTape(NV, 0, NV.tapeConfig.size()-1);

	int totalWs = 0;
	int currentLocation = 0;
	int currentLocationTemp = 0;
	while(totalEnd.compareNextChunk(W, currentLocationTemp))
	{
		totalWs++;
		currentLocation += NV.tapeConfig.size();
		currentLocationTemp = currentLocation;
	}
	Vend.fillTape(totalEnd, currentLocation, totalEnd.tapeConfig.size()-1);
	return totalWs;
}

bool checkNestedReturn(turing_machine<d_type> &machine,
			unsigned int secondRightBound,
			unsigned int thirdRightBound,
			unsigned int transition_limit,
			std::vector<unsigned int> rightBoundCounters,
			unsigned int NVWidth,
			tapeChunk<d_type> &U,
			tapeChunk<d_type> &X,
			tapeChunk<d_type> &V,
			tapeChunk<d_type> &Up,
			tapeChunk<d_type> &Y,
			tapeChunk<d_type> &Vp,
			std::vector<state<d_type> >::size_type s,
			std::vector<state<d_type> >::size_type q,
			std::vector<state<d_type> >::size_type r)
{
	machine.resetMachine();
	unsigned int step_counter = 0;
	tapeChunk<d_type> NU, NUp, NV, NVp, NVpp, NX, NY, NZ, W, Vend, Vpp, temp, temp2, temp3;
	std::vector<state<d_type> >::size_type ns, nq, nr;

	unsigned int nestedReturnStart = rightBoundCounters[0];

	std::list<d_type>::iterator originalLeftBound, originalRightBound;
	originalLeftBound = machine.access_tape_read_write().currentLocation();
	originalRightBound = machine.access_tape_read_write().currentLocation();
	establishSweep(machine, originalLeftBound, originalRightBound, step_counter, nestedReturnStart+1, transition_limit);

	std::list<d_type>::iterator leftBound, rightBound;
	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int i = 0; i < ((int)NVWidth)-1; i++)
		leftBound--;

	ns = machine.get_current_state();
	NV.fillTape(machine.access_tape_read_write(), leftBound, rightBound);
	std::list<d_type>::iterator tempNUBound = leftBound;
	tempNUBound--;
	NU.fillTape(machine.access_tape_read_write(), originalLeftBound, tempNUBound);

	tapeChunk<d_type> nestedTree = NU+NV;
	int offset = nestedTree.tapeConfig.size() - Up.tapeConfig.size();

	int totalWs = extractws(NV, Y, Vp, W, Vend, offset);

	/*std::cout << "ns   : " << ns << std::endl;
	std::cout << "NV   : "; NV.print();
	std::cout << "W    : "; W.print();
	std::cout << "Vend : "; Vend.print();
	std::cout << "W's  : " << totalWs << std::endl;*/

	for(int i = 0; i <= totalWs; i++)
	{
		//NVns -> nqNV
		leftBound = machine.access_tape_read_write().currentLocation();
		rightBound = machine.access_tape_read_write().currentLocation();
		for(int j = 0; j < NV.tapeConfig.size(); j++)
			leftBound--;
		temp = getNextChunk(machine,
					leftBound,
					rightBound,
					secondRightBound,
					step_counter,
					transition_limit,
					true,				//stop at left
					false,				//stop at right
					false,				//inclusive left
					true,				//inclusive right
					false,				//push left
					false);				//push right
		if(temp.isEmpty()) return false;
		if(i == 0)
		{
			nq = machine.get_current_state();
			NVp.fillTape(temp, 0, NV.tapeConfig.size()-1);
		}
		else
		{
			if(nq != machine.get_current_state())
				return false;
			temp2.emptyTape();
			temp2.fillTape(temp, 0, NV.tapeConfig.size()-1);
			if(temp2 != NVp)
				return false;
		}

		for(int w = 0; w < i; w++)
		{
			//NXnq -> nqNY
			leftBound = machine.access_tape_read_write().currentLocation();
			rightBound = machine.access_tape_read_write().currentLocation();
			for(int j = 0; j < NX.tapeConfig.size(); j++)
				leftBound--;
			temp = getNextChunk(machine,
						leftBound,
						rightBound,
						secondRightBound,
						step_counter,
						transition_limit,
						true,				//stop at left
						false,				//stop at right
						false,				//inclusive left
						true,				//inclusive right
						false,				//push left
						false);				//push right
			if(temp.isEmpty()) return false;
			if(i == 1 && w == 0)
			{
				if(nq != machine.get_current_state())
					return false;
				NY.fillTape(temp, 0, NX.tapeConfig.size()-1);
			}
			else
			{
				if(nq != machine.get_current_state())
					return false;
				temp2.emptyTape();
				temp2.fillTape(temp, 0, NX.tapeConfig.size()-1);
				if(temp2 != NY)
					return false;
			}
		}

		//0*NUnq -> 0*NU'nr
		leftBound = machine.access_tape_read_write().currentLocation();
		rightBound = machine.access_tape_read_write().currentLocation();
		rightBound++;
		temp = getNextChunk(machine,
					leftBound,
					rightBound,
					secondRightBound,
					step_counter,
					transition_limit,
					false,				//stop at left
					true,				//stop at right
					true,				//inclusive left
					false,				//inclusive right
					true,				//push left
					false);				//push right
		if(temp.isEmpty()) return false;
		if(i == 0)
		{
			nr = machine.get_current_state();
			NUp.fillTape(temp, 0, temp.tapeConfig.size()-1);
		}
		else
		{
			if(nr != machine.get_current_state())
				return false;
			temp2.emptyTape();
			temp2.fillTape(temp, 0, temp.tapeConfig.size()-1);
			if(temp2 != NUp)
				return false;
		}

		for(int w = 0; w < i; w++)
		{
			//nrNY -> NZnr
			leftBound = machine.access_tape_read_write().currentLocation();
			rightBound = machine.access_tape_read_write().currentLocation();
			for(int j = 0; j < NY.tapeConfig.size(); j++)
				rightBound++;
			temp = getNextChunk(machine,
						leftBound,
						rightBound,
						secondRightBound,
						step_counter,
						transition_limit,
						false,				//stop at left
						true,				//stop at right
						true,				//inclusive left
						false,				//inclusive right
						false,				//push left
						false);				//push right
			if(temp.isEmpty()) return false;
			if(i == 1 && w == 0)
			{
				if(nr != machine.get_current_state())
					return false;
				NZ.fillTape(temp, 0, NY.tapeConfig.size()-1);
			}
			else
			{
				if(nr != machine.get_current_state())
					return false;
				temp2.emptyTape();
				temp2.fillTape(temp, 0, NY.tapeConfig.size()-1);
				if(temp2 != NZ)
					return false;
			}
		}

		if(i != totalWs)
		{
			if(i >= rightBoundCounters.size()-1)
				return false;
			//nrNV' -> NVns''
			leftBound = machine.access_tape_read_write().currentLocation();
			rightBound = machine.access_tape_read_write().currentLocation();
			for(int j = 0; j < NVp.tapeConfig.size(); j++)
				rightBound++;
			temp = getNextChunk(machine,
						leftBound,
						rightBound,
						rightBoundCounters[i+1],
						step_counter,
						transition_limit,
						false,				//stop at left
						false,				//stop at right
						true,				//inclusive left
						true,				//inclusive right
						false,				//push left
						true);				//push right
			if(temp.isEmpty()) return false;
			if(i == 0)
			{
				if(ns != machine.get_current_state())
					return false;
				NVpp.fillTape(temp, 0, temp.tapeConfig.size()-1);
				temp2.emptyTape();
				temp2 = NUp+NVpp;
				temp3 = NU+NV;
				int xLength = temp2.tapeConfig.size() - temp3.tapeConfig.size();
				int xStart = 0;
				temp2.compareNextChunk(NU, xStart);

				if(xStart+xLength-1 < temp2.tapeConfig.size())
					NX.fillTape(temp2, xStart, xStart+xLength-1);
				else return false;
			}
			else
			{
				if(ns != machine.get_current_state())
					return false;
				temp2.emptyTape();
				temp2.fillTape(temp, 0, temp.tapeConfig.size()-1);
				if(temp2 != NVpp)
					return false;

				tapeChunk<d_type> Zs = NZ, Xs = NX;
				for(int z = 0; z < i-1; z++)
					Zs = Zs + NZ;
				for(int x = 0; x < i; x++)
					Xs = Xs + NX;
				if(NUp+Zs+NVpp != NU+Xs+NV)
					return false;
			}
		}
		else
		{
			//nrNV' -> NVs''
			leftBound = machine.access_tape_read_write().currentLocation();
			rightBound = machine.access_tape_read_write().currentLocation();
			for(int j = 0; j < NVp.tapeConfig.size(); j++)
				rightBound++;
			temp = getNextChunk(machine,
						leftBound,
						rightBound,
						secondRightBound,
						step_counter,
						transition_limit,
						false,				//stop at left
						false,				//stop at right
						true,				//inclusive left
						true,				//inclusive right
						false,				//push left
						true);				//push right
			if(temp.isEmpty()) return false;
			if(s != machine.get_current_state())
				return false;
			Vpp.fillTape(temp, 0, temp.tapeConfig.size()-1);
			tapeChunk<d_type> Zs = NZ;
			for(int z = 0; z < i-1; z++)
				Zs = Zs + NZ;

			if(NUp+Zs+Vpp != U+X+X+X+V)
				return false;
		}
	}

	/*std::cout << "NU   : "; NU.print();
	std::cout << "NUp  : "; NUp.print();
	std::cout << "NV   : "; NV.print();
	std::cout << "NVp  : "; NVp.print();
	std::cout << "NVpp : "; NVpp.print();
	std::cout << "NX   : "; NX.print();
	std::cout << "NY   : "; NY.print();
	std::cout << "NZ   : "; NZ.print();
	std::cout << "W    : "; W.print();
	std::cout << "Vend : "; Vend.print();*/

	return true;
}

bool checkNestedChristmasTreeGrammar(turing_machine<d_type> &machine,
							   unsigned int firstRightBound,
							   unsigned int secondRightBound,
							   unsigned int thirdRightBound,
							   unsigned int transition_limit,
							   tapeChunk<d_type> &U,
							   tapeChunk<d_type> &X,
							   tapeChunk<d_type> &V,
							   std::vector<state<d_type> >::size_type s)
{
	machine.resetMachine();
	unsigned int step_counter = 0;
	tapeChunk<d_type> Y, Vp, Up, temp;
	std::vector<state<d_type> >::size_type q, r;

	for(step_counter; step_counter <= firstRightBound; step_counter++)
		machine.run_one();

	std::list<d_type>::iterator leftBound, rightBound;

	//XVs0* -> qYV'0*
	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
	for(int j = 0; j < V.tapeConfig.size() + X.tapeConfig.size(); j++)
		leftBound--;
	temp = getNextChunk(machine,
						leftBound,
						rightBound,
						secondRightBound,
						step_counter,
						transition_limit,
						true,				//stop at left
						false,				//stop at right
						false,				//inclusive left
						true,				//inclusive right
						false,				//push left
						false);				//push right
	if(temp.isEmpty()) return false;
	q = machine.get_current_state();
	Y.fillTape(temp, 0, X.tapeConfig.size()-1);
	Vp.fillTape(temp, X.tapeConfig.size(), temp.tapeConfig.size()-1);

	//0*Uq -> 0*U'r
	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
		rightBound++;
	temp = getNextChunk(machine,
						leftBound,
						rightBound,
						secondRightBound,
						step_counter,
						transition_limit,
						false,				//stop at left
						true,				//stop at right
						true,				//inclusive left
						false,				//inclusive right
						true,				//push left
						false);				//push right
	if(temp.isEmpty()) return false;
	r = machine.get_current_state();
	Up.fillTape(temp, 0, temp.tapeConfig.size()-1);

	leftBound = machine.access_tape_read_write().currentLocation();
	rightBound = machine.access_tape_read_write().currentLocation();
	leftBound--;
	unsigned int nestedReturnStart;
	/*while(machine.access_tape_read_write().currentLocation() != leftBound)
	{
		if(machine.access_tape_read_write().currentLocation() == rightBound)
		{
			if(machine.nextMove() ==  infinite_tape<d_type>::right)
			{
				nestedReturnStart = step_counter;
				machine.access_tape_read_write().checkIteratorRight(rightBound);
				rightBound++;
			}
		}
		machine.run_one();
		step_counter++;
		if(step_counter == transition_limit)
			return false;
	}*/

	std::vector<unsigned int> rightBoundCounters;

	unsigned int test;
	nestedReturnStart = getNextNestedRightBound(machine, rightBound, leftBound, step_counter, secondRightBound);
	rightBoundCounters.push_back((unsigned int)nestedReturnStart);

	std::list<d_type>::iterator oldRightBound = rightBound;
	test = getNextNestedRightBound(machine, rightBound, leftBound, step_counter, secondRightBound);
	int NVWidth = 0;
	while(oldRightBound != rightBound)
	{
		NVWidth++;
		oldRightBound++;
		if(NVWidth > 1000)
			return false;
	}
	while(test != 0)
	{
		rightBoundCounters.push_back(test);
		test = getNextNestedRightBound(machine, rightBound, leftBound, step_counter, secondRightBound);
	}

	/*std::cout << "Up : "; Up.print();
	std::cout << "Y  : "; Y.print();
	std::cout << "Vp: "; Vp.print();
	std::cout << "U :  "; U.print();
	std::cout << "X :  "; X.print();
	std::cout << "V :  "; V.print();
	std::cout << "nestedReturnStart : " << nestedReturnStart << std::endl;*/

	if(checkNestedReturn(machine, secondRightBound, thirdRightBound, transition_limit, rightBoundCounters, NVWidth, U, X, V, Up, Y, Vp, s, q, r))
		return true;
	return false;
}

bool checkNestedChristmasTree(turing_machine<d_type> &machine, unsigned int transition_limit)
{
	machine.resetMachine();
	unsigned int step_counter = 0;

	std::list<d_type>::iterator leftBound;
	std::list<d_type>::iterator rightBound;

	//last bound push
	//0 = left
	//1 = right
	int lastPush = 0;

	//run for a bit to establish sweeping motion
	if(!establishSweep(machine, leftBound, rightBound, step_counter, 150, transition_limit))
		return false;

	//get the step counters of the left and right bounds
	unsigned int firstRightBound = getNextNestedRightBound(machine, rightBound, leftBound, step_counter, transition_limit);
	if(firstRightBound == 0)
		return false;

	unsigned int secondRightBound = getNextNestedRightBound(machine, rightBound, leftBound, step_counter, transition_limit);
	if(secondRightBound == 0)
		return false;

	unsigned int thirdRightBound = getNextNestedRightBound(machine, rightBound, leftBound, step_counter, transition_limit);
	if(thirdRightBound == 0)
		return false;

	unsigned int fourthRightBound = getNextNestedRightBound(machine, rightBound, leftBound, step_counter, transition_limit);
	if(fourthRightBound == 0)
		return false;

	//std::cout << "here" << std::endl;

	//reset the machine and use step counters to get tape configuration at each point
	machine.resetMachine();
	tapeChunk<d_type> beforeSweep, afterSweep;

	std::vector<state<d_type> >::size_type stateS;

	for(unsigned int j = 0; j <= secondRightBound; j++)
	{
		if(j < 50)
			machine.run_one();
		if(j == 50)
			leftBound = machine.access_tape_read_write().currentLocation();
		if(j >= 50)
		{
			if(machine.access_tape_read_write().currentLocation() == leftBound)
				if(machine.nextMove() ==  infinite_tape<d_type>::left)
				{
					machine.access_tape_read_write().checkIteratorLeft(leftBound);
					leftBound--;
				}
			machine.run_one();

			if(j == firstRightBound)
			{
				rightBound = machine.access_tape_read_write().currentLocation();
				beforeSweep.fillTape(machine.access_tape_read_write(), leftBound, rightBound);
				stateS = machine.get_current_state();
			}

			if(j == secondRightBound)
			{
				rightBound = machine.access_tape_read_write().currentLocation();
				afterSweep.fillTape(machine.access_tape_read_write(), leftBound, rightBound);
				if(machine.get_current_state() != stateS)
				{
					std::cout << stateS << " " << machine.get_current_state() << std::endl;
					return false;
				}
			}
		}
	}
	//std::cout << "here2" << std::endl;


	tapeChunk<d_type> chunkX, chunkU, chunkV;

	if(!findMidSection(beforeSweep, afterSweep, chunkX, chunkU, chunkV))
		return false;

	chunkX.print();
	chunkU.print();
	chunkV.print();

	if(!checkNestedChristmasTreeGrammar(machine, secondRightBound, thirdRightBound, fourthRightBound, transition_limit, chunkU, chunkX, chunkV, stateS))
		return false;



	return true;
}

bool nestedChristmasTree(turing_machine<d_type> &machine, unsigned int transition_limit)
{
	if(checkNestedChristmasTree(machine, transition_limit))
		return true;
	else
	{
		turing_machine<d_type> reverse = machine.reverseMachine();
		if(checkNestedChristmasTree(reverse, transition_limit))
			return true;
	}
	return false;
}

//end nested christmas tree routines

#endif
