#ifndef ross_state_header
#define ross_state_header

/*Authored 2002    by  Kyle  Ross,  Undergraduate  Philosophy   Thesis
(Advisor: Professor Bram van Heuveln)


This  file is part of the   "Generic Turing Machine Implementation for
Use in the Exploration of the Busy Beaver Problem".

The "Generic Turing Machine Implementation for  Use in the Exploration
of the Busy Beaver Problem" is free  software; you can redistribute it
and/or modify it under the terms of the  GNU General Public License as
published   by the Free Software Foundation;   either version 2 of the
License, or (at your option) any later version.

This programme is distributed in the hope that it  will be useful, but
WITHOUT    ANY  WARRANTY;  without   even  the    implied warranty  of
MERCHANTABILITY or FITNESS FOR  A  PARTICULAR  PURPOSE.  See  the  GNU
General Public License for more details.

You should have received   a copy of  the  GNU General Public  License
along  with  this   programme; if  not,  write   to the Free  Software
Foundation, Inc., 59  Temple Place, Suite  330, Boston, MA  02111-1307
USA (or visit http://www.fsf.org/licenses/licenses.html)

Created as part of the Busy Beaver Project:
Bram van Heuveln <heuveb@rpi.edu>
Boleshaw Szymanski <szymansk@cs.rpi.edu>
Selmer Bringsjord <selmer@rpi.edu>
Carlos Varela <cvarela@cs.rpi.edu>
Owen Kellett <kelleo@rpi.edu>
Shailesh Kelkar <kelkas@rpi.edu>
Kyle Ross <rossk2@rpi.edu>
Rensselaer Reasoning Group
Department of Cognitive Science
Department of Computer Science
Rensselaer Polytechnic Institute
Troy, New York 12180
U.S.A.
*/
//implements a finite-state automaton suitable for use in implementation of a four-tuple turing machine

#include <map>
#include <vector>
#include "debug.hpp"
#include "infinite_tape.hpp"

/**
 * Single state of turing machine.
 * @ingroup TuringMachineDefinition
 */
template <typename T>
class state
{
public:
	enum transition_result{run, halt};

	/**
	 * Default constructor.
	 * @author Kyle Ross
	 */
	state()
	{
		#ifdef ross_debug_mode
			std::cout<<"["<<this<<"]state constructed"<<std::endl;
		#endif
	}
	
	/**
	 * trivial destructor.
	 * @author Kyle Ross
	 */
	~state()
	{
		#ifdef ross_debug_mode
			std::cout<<"["<<this<<"]state destructed"<<std::endl;
		#endif
	}

	//user functionality associated with making transitions
	//create various transition types
	/**
	 * adds a full transition (read, write, move, next_state).
	 * adds a full 5 tuple transition
	 * @post the requested transition has been added, overwriting the currently-existing transition for read
	 * @param read symbol read on the tape
	 * @param write symbol to be written on the tape
	 * @param next_state next state to transition to
	 * @param move direction to move the read head
	 * @author Kyle Ross
	 */
	void create_transition(T read, T write, typename std::vector<state<T> >::size_type next_state, typename infinite_tape<T>::direction move)
	{
		#ifdef kyle_debug_mode
			if(transition_map.find(transition_struct(write, next_state, move))!=transition_map.end())
			{
				std::cout<<"["<<this<<"]state::create-transition WARNING: transition has been replaced"<<std::endl;
			}
		#endif
		transition_map[read]=transition_struct(write, next_state, move);
	}

	/**
	 * adds a full transition (read, write, next_state)
	 * adds a 4 tuple write transition
	 * @post the requested transition has been added, overwriting the currently-existing transition for read
	 * @param read symbol read on the tape
	 * @param write symbol to be written to the tape
	 * @param next_state next state to transition to
	 * @author Kyle Ross
	 */
	void create_write_transition(T read, T write, typename std::vector<state<T> >::size_type next_state)
	{
		create_transition(read, write, next_state, infinite_tape<T>::do_nothing);
	}

	/**
	 * adds a full transition (read, move, next_state)
	 * adds a 4 tuple move transition
	 * @post the requested transition has been added, overwriting the currently-existing transition for read
	 * @param read symbol read on the tape
	 * @param next_state next state to transition to
	 * @param move direction to move the read head
	 * @author Kyle Ross
	 */
	void create_move_transition(T read, typename std::vector<state<T> >::size_type next_state, typename infinite_tape<T>::direction move)
	{
		create_transition(read, read, next_state, move);
	}

	/**
	 * performs a transition given a tape.
	 * @pre tape.read(), tape.write(), tape.move(left), tape.move(right) are valid operations
	 * @post transition has been processed, updating current state and tape
	 * @param tape tape to perform the transition on
	 * @param state_index reference to the current state which will be updated
	 * @return run if success, halt if failure
	 * @author Kyle Ross
	 */
	transition_result perform_transition(infinite_tape<T> &tape, typename std::vector<state<T> >::size_type &state_index)
	{
		typename std::map<T, transition_struct>::iterator lookup_result(transition_map.find(tape.read()));
		if(lookup_result==transition_map.end())//no transition, halt
		{
			return halt; //return failure
		}
		else //process the transition
		{
			tape.write((*lookup_result).second.write); //write new symbol to tape
			tape.move((*lookup_result).second.move); //move tape's head
			state_index=(*lookup_result).second.next_state; //set the next state
			return run; //return success
		}
	}

	/**
	 * simulates performance of a transition given the relevant tape specification.
	 * @pre the parameters tape_read holds the symbol that would have been read
	 * @post tape_write, tape_move, state_index are set to reflect the operations that would have been taken by perform_transition and the return value is the same of perform_condition
	 * @param tape_read symbol to simulate transition on
	 * @param tape_write updated to symbol that would've been written
	 * @param tape_move updated to direction that would've been moved
	 * @param state_index updated to state that would've been transitioned to
	 * @return run if success, halt if failure
	 * @author Kyle Ross
	 */
	transition_result simulate_transition(T tape_read, T &tape_write, typename infinite_tape<T>::direction &tape_move, typename std::vector<state<T> >::size_type &state_index) const
	{
		//lookup the transition for the current read symbol
		typename std::map<T, transition_struct>::const_iterator lookup_result=transition_map.find(tape_read);
		if(lookup_result==transition_map.end())//no transition, halt
		{
			return halt;
		}
		else //process the transition
		{
			tape_write=(*lookup_result).second.write; //the new tape symbol that would have been written
			tape_move=(*lookup_result).second.move; //the head's move that would have been performed
			state_index=(*lookup_result).second.next_state; //the state that would have been entered
			return run; //return success
		}
	}
	//displays the states of the set
	//preconditions: there are zero or more states defined
	//postconditions: "  < no transitions defined >" has been displayed if there are none defined, else each transition has been displayed in the format "  read("<symbol>") -> "<transition-struct-format-output><endline>
	//complexity: O(NM) where N is the number of states defined and M is the alphabet size
	/**
	 * displayes the states of the set.
	 * @pre there are zero or more states defined
	 * @post "  < no transitions defined >" has been displayed if there are none defined, else each transition has been displayed in the format "  read("<symbol>") -> "<transition-struct-format-output><endline>
	 * @author Kyle Ross
	 */
	void print(std::ostream & out) const
	{
		if(transition_map.begin()==transition_map.end())
		{
			out<<"  < no transitions defined >"<<std::endl;
		}
		else
		{
			for(typename std::map<T, transition_struct>::const_iterator i=transition_map.begin(); i!=transition_map.end(); ++i)
			{
				out<<"  read("<<(*i).first<<") -> ";
				(*i).second.print(out);
				out<<std::endl;
			}
		}
	}
protected:
	/**
	 * structure to hold transition.
	 */
	struct transition_struct
	{
		/**
		 * default constructor.
		 * @author Kyle Ross
		 */
		transition_struct()
		{}

		/**
		 * construct a transition_struct from the proper data members
		 * @param new_write write symbol
		 * @param new_next_state next state to transition to
		 * @param new_move direction to move
		 * @author Kyle Ross
		 */
		transition_struct(T new_write, typename std::vector<state<T> >::size_type new_next_state, typename infinite_tape<T>::direction new_move)
		:write(new_write), next_state(new_next_state), move(new_move)
		{}

		/**
		 * displays a transition.
		 * @pre the state has printable values
		 * @post the state has been displayed in the format "write("<write-value>") move("<left-or-right-or-do_nothing>") nextstate("<next-state-ID-num>")"
		 * @author Kyle Ross
		 */
		void print(std::ostream &out) const
		{
			out<<"write("<<write<<") move(";
			switch(move)
			{
				case infinite_tape<T>::left:
					out<<"left";
					break;
				case infinite_tape<T>::right:
					out<<"right";
					break;
				case infinite_tape<T>::do_nothing:
					out<<"do_nothing";
			}
			out<<") next_state("<<next_state<<")";
		}
		T write;
		typename std::vector<state<T> >::size_type next_state;
		typename infinite_tape<T>::direction move;

	};
	/**
	 * map of the transitions on this state with the read symbol as the key
	 */
	typename std::map<T, transition_struct> transition_map;
};


/**
 * displays a state set.
 * @pre to_print is a valid state set
 * @post to_print has been displayed according to the rules of the print member function and out has been returned
 * @author Kyle Ross
 * @ingroup TuringMachineDefinition
 */
template <typename T>
std::ostream& operator<<(std::ostream &out, const state<T> &to_print)
{
	to_print.print(out);
	return out;
}

#endif


